@@ -73,7 +73,7 @@ newGRanges <- ## hidden constructor shared with other GRanges-like objects
     if (!is.factor(runValue(strand)) ||
         !identical(levels(runValue(strand)), levels(strand())))
         runValue(strand) <- strand(runValue(strand))
-    if (IRanges:::anyMissing(runValue(strand))) {
+    if (S4Vectors:::anyMissing(runValue(strand))) {
         warning("missing values in strand converted to \"*\"")
         runValue(strand)[is.na(runValue(strand))] <- "*"
     }
@@ -98,21 +98,20 @@ newGRanges <- ## hidden constructor shared with other GRanges-like objects
     ## in case we have seqlengths for unrepresented sequences
     runValue(seqnames) <- factor(runValue(seqnames), seqnames(seqinfo))
 
-    if (!is.null(mcols(ranges))) {
-        warning("'ranges' has metadata columns, dropping them")
-        mcols(ranges) <- NULL
-    }
-
     if (!is(mcols, "DataFrame"))  # should never happen when calling GRanges()
         stop("'mcols' must be a DataFrame object")
     if (ncol(mcols) == 0L) {
-        mcols <- new("DataFrame", nrows = length(seqnames))
-    } else if (!is.null(rownames(mcols))) {
+        mcols <- mcols(ranges)
+        if (is.null(mcols))
+            mcols <- new("DataFrame", nrows=length(seqnames))
+    }
+    if (!is.null(mcols(ranges)))
+        mcols(ranges) <- NULL
+    if (!is.null(rownames(mcols))) {
         if (is.null(names(ranges)))
             names(ranges) <- rownames(mcols)
         rownames(mcols) <- NULL
     }
-
     new(class, seqnames = seqnames, ranges = ranges, strand = strand,
         seqinfo = seqinfo, elementMetadata = mcols)
 }
@@ -194,6 +193,9 @@ setAs("RangesList", "GRanges",
         ranges <- unlist(from, use.names=FALSE)
         ranges <- IRanges(start=start(ranges), width=width(ranges))
         ## From now, ranges is guaranteed to be an IRanges *instance*.
+        if (is.null(space(from))) {
+          stop("Cannot create GRanges when 'space(from)' is NULL")
+        }
         gr <- GRanges(seqnames = space(from),
                       ranges = ranges,
                       strand = Rle("*", length(ranges)))
@@ -222,7 +224,17 @@ setAs("Seqinfo", "GRanges", .fromSeqinfoToGRanges)
 ###
 
 setMethod("seqnames", "GRanges", function(x) x@seqnames)
-setMethod("ranges", "GRanges", function(x, ...) x@ranges)
+setMethod("ranges", "GRanges",
+    function(x, use.mcols=FALSE)
+    {
+        if (!isTRUEorFALSE(use.mcols))
+            stop("'use.mcols' must be TRUE or FALSE")
+        ans <- x@ranges
+        if (use.mcols)
+            mcols(ans) <- mcols(x)
+        ans
+    }
+)
 setMethod("strand", "GRanges", function(x) x@strand)
 setMethod("seqinfo", "GRanges", function(x) x@seqinfo)
 
