@@ -54,11 +54,11 @@ setValidity2("GRangesList", .valid.GRangesList)
 GRangesList <- function(...)
 {
     listData <- list(...)
+    if (length(listData) == 1L && is.list(listData[[1L]]))
+        listData <- listData[[1L]]
     if (length(listData) == 0L) {
         unlistData <- GRanges()
     } else {
-        if (length(listData) == 1L && is.list(listData[[1L]]))
-            listData <- listData[[1L]]
         if (!all(sapply(listData, is, "GRanges")))
             stop("all elements in '...' must be GRanges objects")
         unlistData <- suppressWarnings(do.call("c", unname(listData)))
@@ -212,14 +212,14 @@ setMethod("start", "GRangesList",
 )
 
 setReplaceMethod("start", "GRangesList",
-    function(x, check = TRUE, value)
+    function(x, ..., value)
     {
         if (!is(value, "IntegerList") ||
             !identical(elementLengths(x), elementLengths(value)))
             stop("replacement 'value' is not an IntegerList with the same ",
                  "elementLengths as 'x'")
         value <- unlist(value, use.names = FALSE)
-        start(ranges(x@unlistData), check = check) <- value
+        start(x@unlistData, ...) <- value
         x
     }
 )
@@ -234,14 +234,14 @@ setMethod("end", "GRangesList",
 )
 
 setReplaceMethod("end", "GRangesList",
-    function(x, check = TRUE, value)
+    function(x, ..., value)
     {
         if (!is(value, "IntegerList") ||
             !identical(elementLengths(x), elementLengths(value)))
             stop("replacement 'value' is not an IntegerList with the same ",
                  "elementLengths as 'x'")
         value <- unlist(value, use.names = FALSE)
-        end(ranges(x@unlistData), check = check) <- value
+        end(x@unlistData, ...) <- value
         x
     }
 )
@@ -256,14 +256,14 @@ setMethod("width", "GRangesList",
 )
 
 setReplaceMethod("width", "GRangesList",
-    function(x, check = TRUE, value)
+    function(x, ..., value)
     {
         if (!is(value, "IntegerList") ||
             !identical(elementLengths(x), elementLengths(value)))
             stop("replacement 'value' is not an IntegerList with the same ",
                  "elementLengths as 'x'")
         value <- unlist(value, use.names = FALSE)
-        width(ranges(x@unlistData), check = check) <- value
+        width(x@unlistData, ...) <- value
         x
     }
 )
@@ -292,7 +292,16 @@ replaceStrandList <- function(x, value)
     strand(x@unlistData) <- value
     x
 }
-setReplaceMethod("strand", "GRangesList", replaceStrandList)
+setReplaceMethod("strand", c("GRangesList", "ANY"), replaceStrandList)
+setReplaceMethod("strand", c("GRangesList", "character"), 
+    function(x, ..., value)
+    {
+        if (length(value) > 1L)
+            stop("length(value) must be 1")
+        strand(x@unlistData) <- value
+        x
+    }
+)
 
 ### NOT exported but used in GenomicAlignments package.
 getElementMetadataList <- 
@@ -359,7 +368,7 @@ replaceSeqinfoList <- function(x, new2old=NULL, force=FALSE, value)
 {
     if (!is(value, "Seqinfo"))
         stop("the supplied 'seqinfo' must be a Seqinfo object")
-    dangling_seqlevels <- getDanglingSeqlevels(x,
+    dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
                               new2old=new2old, force=force,
                               seqlevels(value))
     if (length(dangling_seqlevels) != 0L) {
@@ -385,22 +394,6 @@ setReplaceMethod("score", "GRangesList", function(x, value) {
 ### Coercion.
 ###
 
-### NOT exported but used in GenomicAlignments package.
-GRangesListAsdataframe <- function(x, row.names=NULL, optional=FALSE, ...)
-{
-    if (missing(row.names))
-        row.names <- names(x@unlistData)
-    if (is.null(names(x)))
-        element <- rep.int(seq_len(length(x)), elementLengths(x))
-    else
-        element <- rep.int(names(x), elementLengths(x))
-    data.frame(element = element,
-               as.data.frame(unlist(x, use.names = FALSE),
-                             row.names = row.names),
-               stringsAsFactors = FALSE)
-}
-setMethod("as.data.frame", "GRangesList", GRangesListAsdataframe)
-
 setAs("GRangesList", "CompressedIRangesList",
     function(from) ranges(from, use.mcols=TRUE)
 )
@@ -414,6 +407,7 @@ setAs("GRangesList", "RangesList",
 setAs("RangedDataList", "GRangesList",
       function(from) GRangesList(lapply(from, as, "GRanges")))
 
+setAs("GRanges", "GRangesList", function(from) as(from, "List"))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
@@ -446,7 +440,7 @@ setMethod("[", "GRangesList", .sBracketSubsetGRList)
     if (!is(value, class(x)[1]))
         stop(paste0("replacement value must be a ", class(x)[1], " object"))
     if (!missing(i))
-        i <- IRanges:::extractROWS(setNames(seq_along(x), names(x)), i)
+        i <- extractROWS(setNames(seq_along(x), names(x)), i)
     if (!missing(j)) {
         if (!is.character(j))
             stop("'j' must be a character vector")
@@ -476,7 +470,7 @@ setReplaceMethod("[", "GRangesList", .sBracketReplaceGRList)
 .dBracketReplaceGRList <- function(x, i, j, ..., value)
 {
     nameValue <- if (is.character(i)) i else ""
-    i <- IRanges:::normargSubset2_iOnly(x, i, j, ...,
+    i <- S4Vectors:::normargSubset2_iOnly(x, i, j, ...,
              .conditionPrefix=paste0("[[<-,", class(x)[1], "-method: "))
     len <- length(x)
     if (i > len) {
@@ -499,22 +493,18 @@ setReplaceMethod("[[", "GRangesList", .dBracketReplaceGRList)
 
 setMethod("relistToClass", "GRanges", function(x) "GRangesList")
 
-setMethod("splitAsListReturnedClass", "GRanges",
-    function(x) {.Deprecated("relistToClass"); "GRangesList"}
-)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### show method.
 ###
 
 ### NOT exported but used in GenomicAlignments package.
-showList <- function(object, showFunction, with.classinfo, ...)
+showList <- function(object, showFunction, print.classinfo)
 {
     k <- length(object)
     cumsumN <- cumsum(elementLengths(object))
     N <- tail(cumsumN, 1)
-    cat(class(object), " of length ", k, ":\n", sep = "")
+    cat(class(object), " object of length ", k, ":\n", sep = "")
     if (k == 0L) {
         cat("<0 elements>\n\n")
     } else if ((k == 1L) || ((k <= 3L) && (N <= 20L))) {
@@ -530,9 +520,9 @@ showList <- function(object, showFunction, with.classinfo, ...)
         for (i in seq_len(k)) {
             cat(nms[i], "\n")
             showFunction(object[[i]], margin="  ",
-                         with.classinfo=with.classinfo)
-            if (with.classinfo)
-                with.classinfo <- FALSE
+                         print.classinfo=print.classinfo)
+            if (print.classinfo)
+                print.classinfo <- FALSE
             cat("\n")
         }
     } else {
@@ -556,9 +546,9 @@ showList <- function(object, showFunction, with.classinfo, ...)
         for (i in seq_len(showK)) {
             cat(nms[i], "\n")
             showFunction(object[[i]], margin="  ",
-                              with.classinfo=with.classinfo)
-            if (with.classinfo)
-                with.classinfo <- FALSE
+                         print.classinfo=print.classinfo)
+            if (print.classinfo)
+                print.classinfo <- FALSE
             cat("\n")
         }
         if (diffK > 0) {
@@ -567,8 +557,8 @@ showList <- function(object, showFunction, with.classinfo, ...)
                 sep="")
         }
     }
-    cat("---\n")
-    showSeqlengths(object)
+    cat("-------\n")
+    cat("seqinfo: ", summary(seqinfo(object)), "\n", sep="")
 }
 
 setMethod("show", "GRangesList",
@@ -602,12 +592,12 @@ deconstructGRLintoGR <- function(x, expand.levels=FALSE)
         i1 <- rep(seq_len(length(x)), each=x_nlev)
         i2 <- rep.int(seq_len(x_nlev), length(x))
     } else {
-        oo <- IRanges:::orderIntegerPairs(f1, f2)
+        oo <- S4Vectors:::orderIntegerPairs(f1, f2)
         of1 <- f1[oo]
         of2 <- f2[oo]
-        ## TODO: Add "presorted" method to IRanges:::duplicatedIntegerPairs()
+        ## TODO: Add "presorted" method to S4Vectors:::duplicatedIntegerPairs()
         ## for when the 2 input vectors are already sorted.
-        notdups <- !IRanges:::duplicatedIntegerPairs(of1, of2)
+        notdups <- !S4Vectors:::duplicatedIntegerPairs(of1, of2)
         i1 <- of1[notdups]
         i2 <- of2[notdups]
     }
