@@ -21,53 +21,6 @@ setClassUnion("GenomicRangesORmissing", c("GenomicRanges", "missing"))
 ### update(x) and clone(x) are defined.
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### 2 non-exported low-level helper functions.
-###
-### For the 2 functions below, 'x' must be a GenomicRanges object.
-### They both return a named integer vector where the names are guaranteed
-### to be 'seqlevels(x)'.
-###
-
-minStartPerGRangesSequence <- function(x)
-{
-    cil <- splitAsList(start(x), seqnames(x))  # CompressedIntegerList object
-    ## The 4 lines below are equivalent to:
-    ##   ans <- min(cil)
-    ##   ans[elementLengths(v) == 0L] <- NA_integer_
-    ## but much faster!
-    ## TODO: Replace with the above, but only when the "min" method for
-    ## CompressedIntegerList objects (implemented in the IRanges package)
-    ## is as fast as the "viewMins" method for XIntegerViews objects
-    ## (implemented in C in the XVector package). Ideally, the 2 methods
-    ## should share the same underlying code.
-    v <- Views(cil@unlistData, cil@partitioning)  # XIntegerViews object
-    ans <- viewMins(v)
-    ans[width(v) == 0L] <- NA_integer_
-    names(ans) <- names(v)
-    ans
-}
-
-maxEndPerGRangesSequence <- function(x)
-{
-    cil <- splitAsList(end(x), seqnames(x))  # CompressedIntegerList object
-    ## The 4 lines below are equivalent to:
-    ##   ans <- max(cil)
-    ##   ans[elementLengths(v) == 0L] <- NA_integer_
-    ## but much faster!
-    ## TODO: Replace with the above, but only when the "max" method for
-    ## CompressedIntegerList objects (implemented in the IRanges package)
-    ## is as fast as the "viewMaxs" method for XIntegerViews objects
-    ## (implemented in C in the XVector package). Ideally, the 2 methods
-    ## should share the same underlying code.
-    v <- Views(cil@unlistData, cil@partitioning)  # XIntegerViews object
-    ans <- viewMaxs(v)
-    ans[width(v) == 0L] <- NA_integer_
-    names(ans) <- names(v)
-    ans
-}
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Getters.
 ###
@@ -78,6 +31,7 @@ setMethod("names", "GenomicRanges", function(x) names(ranges(x)))
 
 #setMethod("constraint", "GenomicRanges", function(x) x@constraint)
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Extra column slots (implemented by subclasses)
 ###
@@ -100,6 +54,55 @@ setMethod("fixedColumnNames", "GenomicRanges", function(x) {
   colnames(as.data.frame(new(class(x))))
 })
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### 2 low-level helper functions to deal with out-of-bound ranges.
+###
+
+### Returns index of out-of-bound ranges located on non-circular sequences
+### whose length is not NA. Works on a GenomicRanges or GAlignments object.
+get_out_of_bound_index <- function(x)
+{
+    if (length(x) == 0L)
+        return(integer(0))
+    x_seqnames_id <- as.integer(seqnames(x))
+    x_seqlengths <- unname(seqlengths(x))
+    seqlevel_is_circ <- unname(isCircular(x)) %in% TRUE
+    seqlength_is_na <- is.na(x_seqlengths)
+    seqlevel_has_bounds <- !(seqlevel_is_circ | seqlength_is_na)
+    which(seqlevel_has_bounds[x_seqnames_id] &
+          (start(x) < 1L | end(x) > x_seqlengths[x_seqnames_id]))
+}
+
+### Also works on a GenomicRanges or GAlignments object. Note that GAlignments
+### objects are not trimmable so use 'suggest.trim=FALSE' on them.
+make_out_of_bound_warning_msg <- function(x, idx, suggest.trim)
+{
+    where <- seqlevels(x)[unique(as.integer(seqnames(x))[idx])]
+    if (length(where) == 1L) {
+        on_what <- paste0("sequence ", where)
+    } else if (length(where) == 2L) {
+        on_what <- paste0("sequences ", where[1L], " and ", where[2L])
+    } else {
+        seqlevels_in1string <- paste0(head(where, n=-1L), collapse=", ")
+        on_what <- paste0("sequences ", seqlevels_in1string,
+                              ", and ", tail(where, n=1L))
+    }
+    msg <- c(class(x), " object contains ", length(idx), " out-of-bound ",
+             "range", if (length(idx) >= 2L) "s" else "", " located on ",
+             on_what, ". ",
+             "Note that only ranges located on a non-circular ",
+             "sequence whose length is not NA can be considered ",
+             "out-of-bound (use seqlengths() and isCircular() to ",
+             "get the lengths and circularity flags of the underlying ",
+             "sequences).")
+    if (suggest.trim)
+        msg <- c(msg, " You can use trim() to trim these ranges. ",
+                 "See ?`trim,GenomicRanges-method` for more information.")
+    msg
+}
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity.
 ###
@@ -131,7 +134,7 @@ setMethod("fixedColumnNames", "GenomicRanges", function(x) {
 {
     if (!is.factor(runValue(seqnames(x))))
         return("'seqnames' should be a 'factor' Rle")
-    if (IRanges:::anyMissing(runValue(seqnames(x))))
+    if (S4Vectors:::anyMissing(runValue(seqnames(x))))
         return("'seqnames' contains missing values")
     NULL
 }
@@ -153,7 +156,7 @@ setMethod("fixedColumnNames", "GenomicRanges", function(x) {
                  ")")
         return(paste(msg, collapse=""))
     }
-    if (IRanges:::anyMissing(runValue(strand(x))))
+    if (S4Vectors:::anyMissing(runValue(strand(x))))
         return("'strand' contains missing values")
     NULL
 }
@@ -168,7 +171,7 @@ INVALID.GR.COLNAMES <- c("seqnames", "ranges", "strand",
                          "start", "end", "width", "element")
 
 .valid.GenomicRanges.mcols <- function(x)
-{    
+{
     if (any(INVALID.GR.COLNAMES %in% colnames(mcols(x)))) {
         msg <- c("names of metadata columns cannot be one of ",
                  paste0("\"", INVALID.GR.COLNAMES, "\"", collapse=", "))
@@ -178,7 +181,7 @@ INVALID.GR.COLNAMES <- c("seqnames", "ranges", "strand",
 }
 
 ### Also used by the validity method for GAlignments objects.
-valid.GenomicRanges.seqinfo <- function(x)
+valid.GenomicRanges.seqinfo <- function(x, suggest.trim=FALSE)
 {
     x_seqinfo <- seqinfo(x)
     if (!identical(seqlevels(x_seqinfo), levels(seqnames(x)))) {
@@ -186,28 +189,11 @@ valid.GenomicRanges.seqinfo <- function(x)
                  "are not identical")
         return(paste(msg, collapse=" "))
     }
-    x_seqlengths <- seqlengths(x_seqinfo)
-    seqs_with_known_length <- names(x_seqlengths)[!is.na(x_seqlengths)]
-    if (length(seqs_with_known_length) == 0L)
-        return(NULL)
-    if (any(x_seqlengths < 0L, na.rm=TRUE))
-        return("'seqlengths(x)' contains negative values")
-    ## We check only the ranges that are on a non-circular sequence with
-    ## a known length.
-    x_isCircular <- isCircular(x_seqinfo)
-    non_circ_seqs <- names(x_isCircular)[!(x_isCircular %in% TRUE)]
-    ncswkl <- intersect(non_circ_seqs, seqs_with_known_length)
-    if (length(ncswkl) == 0L)
-        return(NULL)
-    x_seqnames <- seqnames(x)
-    runValue(x_seqnames) <- runValue(x_seqnames)[drop=TRUE]
-    minStarts <- minStartPerGRangesSequence(x)
-    maxEnds <- maxEndPerGRangesSequence(x)
-    if (any(minStarts[ncswkl] < 1L, na.rm=TRUE)
-     || any(maxEnds[ncswkl] >
-            seqlengths(x)[ncswkl], na.rm=TRUE))
-        warning("'ranges' contains values outside of sequence bounds. ",
-                "See ?trim to subset ranges.")
+    idx <- get_out_of_bound_index(x)
+    if (length(idx) != 0L) {
+        msg <- make_out_of_bound_warning_msg(x, idx, suggest.trim)
+        warning(wmsg(msg))
+    }
     NULL
 }
 
@@ -226,7 +212,7 @@ valid.GenomicRanges.seqinfo <- function(x)
       .valid.GenomicRanges.ranges(x),
       .valid.GenomicRanges.strand(x),
       .valid.GenomicRanges.mcols(x),
-      valid.GenomicRanges.seqinfo(x),
+      valid.GenomicRanges.seqinfo(x, suggest.trim=TRUE),
       .valid.GenomicRanges.ecs(x))
       #checkConstraint(x, constraint(x)))
 }
@@ -275,6 +261,7 @@ setAs("GenomicRanges", "RangesList",
     }
 )
 
+### TODO: Turn this into an S3/S4 combo for as.data.frame.GenomicRanges
 setMethod("as.data.frame", "GenomicRanges",
     function(x, row.names=NULL, optional=FALSE, ...)
     {
@@ -311,6 +298,21 @@ setMethod("as.data.frame", "GenomicRanges",
 }
 
 setAs("Seqinfo", "GenomicRanges", .fromSeqinfoToGRanges)
+setAs("Seqinfo", "RangesList",
+    function(from) as(as(from, "GenomicRanges"), "RangesList")
+)
+
+setMethod("granges", "GenomicRanges",
+    function(x, use.mcols=FALSE)
+    {
+        if (!isTRUEorFALSE(use.mcols))
+            stop("'use.mcols' must be TRUE or FALSE")
+        ans <- GRanges(seqnames(x), ranges(x), strand(x), seqinfo=seqinfo(x))
+        if (use.mcols)
+            mcols(ans) <- cbind(extraColumnSlotsAsDF(x), mcols(x))
+        ans
+    }
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -326,7 +328,7 @@ setReplaceMethod("names", "GenomicRanges",
 )
 
 setReplaceMethod("seqnames", "GenomicRanges",
-    function(x, value) 
+    function(x, value)
     {
         if (!is(value, "Rle"))
             value <- Rle(value)
@@ -347,7 +349,7 @@ setReplaceMethod("seqnames", "GenomicRanges",
 )
 
 setReplaceMethod("ranges", "GenomicRanges",
-    function(x, value) 
+    function(x, value)
     {
         if (!is(value, "IRanges"))
             value <- as(value, "IRanges")
@@ -355,7 +357,7 @@ setReplaceMethod("ranges", "GenomicRanges",
         k <- length(value)
         if (k != n) {
             if ((k == 0L) || (k > n) || (n %% k != 0L))
-                stop(k, " elements in value to replace ", n, "elements")
+                stop(k, " elements in value to replace ", n, " elements")
             value <- rep(value, length.out=n)
         }
         update(x, ranges=value, check=FALSE)
@@ -379,7 +381,7 @@ normargGenomicRangesStrand <- function(strand, n)
 }
 
 setReplaceMethod("strand", "GenomicRanges",
-    function(x, value) 
+    function(x, value)
     {
         value <- normargGenomicRangesStrand(value, length(x))
         x <- update(x, strand=value, check=FALSE)
@@ -407,18 +409,19 @@ setReplaceMethod("seqinfo", "GenomicRanges",
     {
         if (!is(value, "Seqinfo"))
             stop("the supplied 'seqinfo' must be a Seqinfo object")
-        dangling_seqlevels <- getDanglingSeqlevels(x,
+        dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
                                   new2old=new2old, force=force,
                                   seqlevels(value))
         if (length(dangling_seqlevels) != 0L)
             x <- x[!(seqnames(x) %in% dangling_seqlevels)]
         old_seqinfo <- seqinfo(x)
-        new_seqnames <- makeNewSeqnames(x, new2old=new2old, seqlevels(value))
+        new_seqnames <- GenomeInfoDb:::makeNewSeqnames(x,
+                                  new2old=new2old, seqlevels(value))
         x <- update(x, seqnames=new_seqnames, seqinfo=value, check=FALSE)
-        geom_has_changed <- sequenceGeometryHasChanged(seqinfo(x), old_seqinfo,
-                                                       new2old=new2old)
+        geom_has_changed <- GenomeInfoDb:::sequenceGeometryHasChanged(
+                                  seqinfo(x), old_seqinfo, new2old=new2old)
         if (any(geom_has_changed, na.rm=TRUE)) {
-            msg <- valid.GenomicRanges.seqinfo(x)
+            msg <- valid.GenomicRanges.seqinfo(x, suggest.trim=TRUE)
             if (!is.null(msg))
                 stop(msg)
         }
@@ -484,62 +487,26 @@ setMethod("end", "GenomicRanges", function(x, ...) end(ranges(x)))
 setMethod("width", "GenomicRanges", function(x) width(ranges(x)))
 
 setReplaceMethod("start", "GenomicRanges",
-    function(x, check=TRUE, value)
+    function(x, ..., value)
     {
-        if (!is.integer(value))
-            value <- as.integer(value)
-        ranges <- ranges(x)
-        starts <- start(ranges)
-        starts[] <- value
-        ## TODO: Revisit this to handle circularity (maybe).
-        if (!IRanges:::anyMissing(seqlengths(x))) {
-            if (IRanges:::anyMissingOrOutside(starts, 1L)) {
-                warning("trimmed start values to be positive")
-                starts[starts < 1L] <- 1L
-            }
-        }
-        start(ranges, check=check) <- starts
-        update(x, ranges=ranges, check=FALSE)
+        new_ranges <- `start<-`(ranges(x), ..., value=value)
+        update(x, ranges=new_ranges, ...)
     }
 )
 
 setReplaceMethod("end", "GenomicRanges",
-    function(x, check=TRUE, value)
+    function(x, ..., value)
     {
-        if (!is.integer(value))
-            value <- as.integer(value)
-        ranges <- ranges(x)
-        ends <- end(ranges)
-        ends[] <- value
-        seqlengths <- seqlengths(x)
-        ## TODO: Revisit this to handle circularity.
-        if (!IRanges:::anyMissing(seqlengths)) {
-            seqlengths <- seqlengths[seqlevels(x)]
-            maxEnds <- seqlengths[as.integer(seqnames(x))]
-            trim <- which(ends > maxEnds)
-            if (length(trim) > 0L) {
-                warning("trimmed end values to be <= seqlengths")
-                ends[trim] <- maxEnds[trim]
-            }
-        }
-        end(ranges, check=check) <- ends
-        update(x, ranges=ranges, check=FALSE)
+        new_ranges <- `end<-`(ranges(x), ..., value=value)
+        update(x, ranges=new_ranges, ...)
     }
 )
 
 setReplaceMethod("width", "GenomicRanges",
-    function(x, check=TRUE, value)
+    function(x, ..., value)
     {
-        if (!is.integer(value))
-            value <- as.integer(value)
-        if (!IRanges:::anyMissing(seqlengths(x))) {
-            end(x) <- start(x) + (value - 1L)
-        } else {
-            ranges <- ranges(x)
-            width(ranges, check=check) <- value
-            x <- update(x, ranges=ranges, check=FALSE)
-        }
-        x
+        new_ranges <- `width<-`(ranges(x), ..., value=value)
+        update(x, ranges=new_ranges, ...)
     }
 )
 
@@ -548,21 +515,22 @@ setReplaceMethod("width", "GenomicRanges",
 ### Subsetting.
 ###
 
-setMethod(IRanges:::extractROWS, "GenomicRanges",
+setMethod("extractROWS", "GenomicRanges",
     function(x, i)
     {
         if (missing(i) || !is(i, "Ranges"))
-            i <- IRanges:::normalizeSingleBracketSubscript(i, x)
-        ans_seqnames <- IRanges:::extractROWS(seqnames(x), i)
-        ans_ranges <- IRanges:::extractROWS(ranges(x), i)
-        ans_strand <- IRanges:::extractROWS(strand(x), i)
-        ans_mcols <- IRanges:::extractROWS(mcols(x), i)
-        ans_ecs <- lapply(extraColumnSlots(x), IRanges:::extractROWS, i)
+            i <- normalizeSingleBracketSubscript(i, x)
+        ans_seqnames <- extractROWS(seqnames(x), i)
+        ans_ranges <- extractROWS(ranges(x), i)
+        ans_strand <- extractROWS(strand(x), i)
+        ans_mcols <- extractROWS(mcols(x), i)
+        ans_ecs <- lapply(extraColumnSlots(x), extractROWS, i)
         clone(x, seqnames=ans_seqnames,
                  ranges=ans_ranges,
                  strand=ans_strand,
                  elementMetadata=ans_mcols,
-                 .slotList=ans_ecs)
+                 .slotList=ans_ecs,
+                 check=FALSE)
     }
 )
 
@@ -572,24 +540,25 @@ setMethod("[", "GenomicRanges",
     {
         if (length(list(...)) > 0L)
             stop("invalid subsetting")
-        ans <- IRanges:::extractROWS(x, i)
+        if (!missing(i))
+            x <- extractROWS(x, i)
         if (missing(j))
-            return(ans)
-        ans_mcols <- mcols(ans)[ , j, drop=FALSE]
-        clone(ans, elementMetadata=ans_mcols)
+            return(x)
+        new_mcols <- mcols(x)[ , j, drop=FALSE]
+        clone(x, elementMetadata=new_mcols, check=FALSE)
     }
 )
 
-setMethod(IRanges:::replaceROWS, "GenomicRanges",
+setMethod("replaceROWS", "GenomicRanges",
     function(x, i, value)
     {
         if (missing(i) || !is(i, "Ranges"))
-            i <- IRanges:::normalizeSingleBracketSubscript(i, x)
+            i <- normalizeSingleBracketSubscript(i, x)
         seqinfo(x) <- merge(seqinfo(x), seqinfo(value))
-        ans_seqnames <- IRanges:::replaceROWS(seqnames(x), i, seqnames(value))
-        ans_ranges <- IRanges:::replaceROWS(ranges(x), i, ranges(value))
-        ans_strand <- IRanges:::replaceROWS(strand(x), i, strand(value))
-        ans_mcols <- IRanges:::replaceROWS(mcols(x), i, mcols(value))
+        ans_seqnames <- replaceROWS(seqnames(x), i, seqnames(value))
+        ans_ranges <- replaceROWS(ranges(x), i, ranges(value))
+        ans_strand <- replaceROWS(strand(x), i, strand(value))
+        ans_mcols <- replaceROWS(mcols(x), i, mcols(value))
         ans_ecs_names <- extraColumnSlotNames(x)
         ans_necs <- length(ans_ecs_names)
         if (ans_necs == 0L) {
@@ -601,8 +570,7 @@ setMethod(IRanges:::replaceROWS, "GenomicRanges",
                 stop("'value' can have more extra column slots but not less")
             ans_ecs <- extraColumnSlotsAsDF(x)
             value_ecs <- extraColumnSlotsAsDF(value)
-            ans_ecs <- IRanges:::replaceROWS(ans_ecs, i,
-                                             value_ecs[seq_len(ans_necs)])
+            ans_ecs <- replaceROWS(ans_ecs, i, value_ecs[seq_len(ans_necs)])
         }
         update(x, seqnames=ans_seqnames,
                   ranges=ans_ranges,
@@ -612,6 +580,8 @@ setMethod(IRanges:::replaceROWS, "GenomicRanges",
     }
 )
 
+### TODO: Refactor to use replaceROWS(). This will make the code much simpler
+### and avoid a lot of duplication with the above "replaceROWS" method.
 setReplaceMethod("[", "GenomicRanges",
     function(x, i, j, ..., value)
     {
@@ -625,7 +595,7 @@ setReplaceMethod("[", "GenomicRanges",
         value_ecs <- extraColumnSlotsAsDF(value)
         x_ecs <- extraColumnSlotsAsDF(x)
         new_ecs <- value_ecs[!names(value_ecs) %in% names(x_ecs)]
-        ecs_to_replace <- intersect(names(value_ecs), names(x_ecs))        
+        ecs_to_replace <- intersect(names(value_ecs), names(x_ecs))
         if (missing(i)) {
             seqnames[] <- seqnames(value)
             ranges[] <- ranges(value)
@@ -638,7 +608,7 @@ setReplaceMethod("[", "GenomicRanges",
                 ans_mcols[names(new_ecs)] <- new_ecs
             x_ecs[ecs_to_replace] <- value_ecs[ecs_to_replace]
         } else {
-            i <- IRanges:::extractROWS(setNames(seq_along(x), names(x)), i)
+            i <- extractROWS(setNames(seq_along(x), names(x)), i)
             seqnames[i] <- seqnames(value)
             ranges[i] <- ranges(value)
             strand[i] <- strand(value)
@@ -785,8 +755,9 @@ setMethod("c", "GenomicRanges",
 
 .makeNakedMatFromGenomicRanges <- function(x)
 {
-    lx <- length(x)
-    nc <- ncol(mcols(x))
+    x_len <- length(x)
+    x_mcols <- mcols(x)
+    x_nmc <- if (is.null(x_mcols)) 0L else ncol(x_mcols)
     ans <- cbind(seqnames=as.character(seqnames(x)),
                  ranges=showAsCell(ranges(x)),
                  strand=as.character(strand(x)))
@@ -795,27 +766,36 @@ setMethod("c", "GenomicRanges",
         ans <- do.call(cbind,
                        c(list(ans), lapply(extraColumnSlots(x), showAsCell)))
     }
-    if (nc > 0L) {
-        tmp <- do.call(data.frame, c(lapply(mcols(x), showAsCell),
+    if (x_nmc > 0L) {
+        tmp <- do.call(data.frame, c(lapply(x_mcols, showAsCell),
                                      list(check.names=FALSE)))
-        ans <- cbind(ans, `|`=rep.int("|", lx), as.matrix(tmp))
+        ans <- cbind(ans, `|`=rep.int("|", x_len), as.matrix(tmp))
     }
     ans
 }
 
 showGenomicRanges <- function(x, margin="",
-                              with.classinfo=FALSE, print.seqlengths=FALSE)
+                              print.classinfo=FALSE, print.seqinfo=FALSE)
 {
-    lx <- length(x)
-    nc <- ncol(mcols(x))
-    cat(class(x), " with ",
-        lx, " ", ifelse(lx == 1L, "range", "ranges"),
+    x_class <- class(x)
+    x_len <- length(x)
+    x_mcols <- mcols(x)
+    x_nmc <- if (is.null(x_mcols)) 0L else ncol(x_mcols)
+    cat(x_class, " object with ",
+        x_len, " ", ifelse(x_len == 1L, "range", "ranges"),
         " and ",
-        nc, " metadata ", ifelse(nc == 1L, "column", "columns"),
+        x_nmc, " metadata ", ifelse(x_nmc == 1L, "column", "columns"),
         ":\n", sep="")
-    out <- IRanges:::makePrettyMatrixForCompactPrinting(x,
+    ## S4Vectors:::makePrettyMatrixForCompactPrinting() assumes that 'x' is
+    ## subsettable but not all GenomicRanges objects are (and if they are,
+    ## subsetting them could be costly). However GRanges objects are assumed
+    ## to be subsettable so if 'x' is not one then we turn it into one (this
+    ## coercion is expected to work on any GenomicRanges object).
+    if (!is(x, "GRanges"))
+        x <- as(x, "GRanges")
+    out <- S4Vectors:::makePrettyMatrixForCompactPrinting(x,
                .makeNakedMatFromGenomicRanges)
-    if (with.classinfo) {
+    if (print.classinfo) {
         .COL2CLASS <- c(
             seqnames="Rle",
             ranges="IRanges",
@@ -823,7 +803,8 @@ showGenomicRanges <- function(x, margin="",
         )
         extraColumnNames <- extraColumnSlotNames(x)
         .COL2CLASS <- c(.COL2CLASS, getSlots(class(x))[extraColumnNames])
-        classinfo <- makeClassinfoRowForCompactPrinting(x, .COL2CLASS)
+        classinfo <-
+            S4Vectors:::makeClassinfoRowForCompactPrinting(x, .COL2CLASS)
         ## A sanity check, but this should never happen!
         stopifnot(identical(colnames(classinfo), colnames(out)))
         out <- rbind(classinfo, out)
@@ -834,16 +815,16 @@ showGenomicRanges <- function(x, margin="",
     ## limit that would typically be reached when 'showHeadLines' global
     ## option is set to Inf.
     print(out, quote=FALSE, right=TRUE, max=length(out))
-    if (print.seqlengths) {
-        cat(margin, "---\n", sep="")
-        showSeqlengths(x, margin=margin)
+    if (print.seqinfo) {
+        cat(margin, "-------\n", sep="")
+        cat(margin, "seqinfo: ", summary(seqinfo(x)), "\n", sep="")
     }
 }
 
 setMethod("show", "GenomicRanges",
     function(object)
         showGenomicRanges(object, margin="  ",
-                          with.classinfo=TRUE, print.seqlengths=TRUE)
+                          print.classinfo=TRUE, print.seqinfo=TRUE)
 )
 
 setMethod("showAsCell", "GenomicRanges",
