@@ -1,4 +1,4 @@
-## 
+##
 ## Helper classes: Assays, ShallowData, ShallowSimpleListAssays
 
 setClass("Assays")
@@ -29,7 +29,7 @@ setMethod("clone", "ShallowData",  # not exported
     xx
 }
 
-## 
+##
 ## SummarizedExperiment
 
 setClass("SummarizedExperiment",
@@ -61,11 +61,11 @@ setClass("SummarizedExperiment",
     NULL
 }
 
-.valid.SummarizedExperiment.rowData_dims <- function(x)
+.valid.SummarizedExperiment.rowRanges_dims <- function(x)
 {
     if (!all(sapply(assays(x, withDimnames=FALSE), nrow) ==
-             length(rowData(x))))
-        return("'rowData' length differs from 'assays' nrow")
+             length(rowRanges(x))))
+        return("'rowRanges' length differs from 'assays' nrow")
     NULL
 }
 
@@ -79,7 +79,7 @@ setClass("SummarizedExperiment",
 
 .valid.SummarizedExperiment.assays_dims <- function(x)
 {
-    c(.valid.SummarizedExperiment.rowData_dims(x),
+    c(.valid.SummarizedExperiment.rowRanges_dims(x),
       .valid.SummarizedExperiment.colData_dims(x))
 }
 
@@ -107,12 +107,23 @@ setGeneric("SummarizedExperiment",
 }
 
 setMethod(SummarizedExperiment, "SimpleList",
-   function(assays, rowData=GRangesList(), colData=DataFrame(),
+   function(assays, rowRanges=GRangesList(), colData=DataFrame(),
              exptData=SimpleList(), ...,
              verbose=FALSE)
 {
-    if (missing(rowData) && 0L != length(assays))
-        rowData <- .GRangesList_assays(assays)
+    extra_args <- list(...)
+    rowData <- extra_args$rowData
+    if (!is.null(rowData)) {
+        if (!missing(rowRanges))
+            stop("both 'rowRanges' and 'rowData' are specified")
+        msg <- c("The 'rowData' argument is deprecated. ",
+                 "Please use 'rowRanges' instead.")
+        .Deprecated(msg=msg)
+        rowRanges <- rowData
+        extra_args$rowData <- NULL
+    } else if (missing(rowRanges) && 0L != length(assays)) {
+        rowRanges <- .GRangesList_assays(assays)
+    }
     if (missing(colData) && 0L != length(assays)) {
         nms <- colnames(assays[[1]])
         if (is.null(nms) && 0L != ncol(assays[[1]]))
@@ -121,7 +132,7 @@ setMethod(SummarizedExperiment, "SimpleList",
     }
 
     FUN <- function(x) {
-        exp <- list(names(rowData), rownames(colData))
+        exp <- list(names(rowRanges), rownames(colData))
         ## dimnames as NULL or list(NULL, NULL)
         all(sapply(dimnames(x), is.null)) ||
             ## or consistent with row / colData
@@ -131,9 +142,12 @@ setMethod(SummarizedExperiment, "SimpleList",
         assays <- endoapply(assays, unname)
     if (!is(assays, "Assays"))
         assays <- .ShallowSimpleListAssays(data=assays)
-
-    new("SummarizedExperiment", exptData=exptData, rowData=rowData,
-        colData=colData, assays=assays, ...)
+    do.call(new, c(list("SummarizedExperiment",
+                        exptData=exptData,
+                        rowData=rowRanges,
+                        colData=colData,
+                        assays=assays),
+                   extra_args))
 })
 
 setMethod(SummarizedExperiment, "missing",
@@ -162,7 +176,7 @@ setMethod(SummarizedExperiment, "matrix",
 setMethod("clone", "SummarizedExperiment",  # not exported
     function(x, ...)
 {
-    ## IRanges:::extraArgsAsList would prevent using clone on
+    ## S4Vectors:::extraArgsAsList would prevent using clone on
     ## subclasses
     args <- list(...)
     firstTime <- TRUE
@@ -201,25 +215,43 @@ setGeneric("exptData", function(x, ...) standardGeneric("exptData"))
 setGeneric("exptData<-",
     function(x, ..., value) standardGeneric("exptData<-"))
 
-## rowData, colData seem too vague, but from eSet derived classes wanted to
+## rowRanges, colData seem too vague, but from eSet derived classes wanted to
 ## call the rows / cols something different from 'features' or 'samples', so
 ## might as well avoid the issue
-setGeneric("rowData", function(x, ...) standardGeneric("rowData"))
+setGeneric("rowRanges", function(x, ...) standardGeneric("rowRanges"))
 
-setGeneric("rowData<-",
-    function(x, ..., value) standardGeneric("rowData<-"))
+setGeneric("rowRanges<-",
+    function(x, ..., value) standardGeneric("rowRanges<-"))
+
+rowData <- function(...)
+{
+    .Deprecated("rowRanges")
+    rowRanges(...)
+}
+
+`rowData<-` <- function(x, ..., value)
+{
+    .Deprecated("rowRanges<-")
+    rowRanges(x, ...) <- value
+}
 
 setGeneric("colData", function(x, ...) standardGeneric("colData"))
 
 setGeneric("colData<-",
     function(x, ..., value) standardGeneric("colData<-"))
 
+setGeneric("assayNames", function(x, ...) standardGeneric("assayNames"))
+
+setGeneric("assayNames<-",
+    function(x, ..., value) standardGeneric("assayNames<-"))
+
 setGeneric("assays",
     function(x, ..., withDimnames=TRUE) standardGeneric("assays"),
     signature="x")
 
 setGeneric("assays<-",
-    function(x, ..., value) standardGeneric("assays<-"))
+    function(x, ..., withDimnames=TRUE, value) standardGeneric("assays<-"),
+    signature=c("x", "value"))
 
 setGeneric("assay", function(x, i, ...) standardGeneric("assay"))
 
@@ -243,24 +275,24 @@ setReplaceMethod("exptData", c("SummarizedExperiment", "list"),
     clone(x, ..., exptData=SimpleList(value))
 })
 
-setMethod(rowData, "SummarizedExperiment",
+setMethod(rowRanges, "SummarizedExperiment",
     function(x, ...) value(x, "rowData"))
 
-.SummarizedExperiment.rowData.replace <-
+.SummarizedExperiment.rowRanges.replace <-
     function(x, ..., value)
 {
     x <- clone(x, ..., rowData=value)
-    msg <- .valid.SummarizedExperiment.rowData_dims(x)
+    msg <- .valid.SummarizedExperiment.rowRanges_dims(x)
     if (!is.null(msg))
         stop(msg)
     x
 }
 
-setReplaceMethod("rowData", c("SummarizedExperiment", "GenomicRanges"),
-    .SummarizedExperiment.rowData.replace)
+setReplaceMethod("rowRanges", c("SummarizedExperiment", "GenomicRanges"),
+    .SummarizedExperiment.rowRanges.replace)
 
-setReplaceMethod("rowData", c("SummarizedExperiment", "GRangesList"),
-    .SummarizedExperiment.rowData.replace)
+setReplaceMethod("rowRanges", c("SummarizedExperiment", "GRangesList"),
+    .SummarizedExperiment.rowRanges.replace)
 
 setMethod(colData, "SummarizedExperiment",
     function(x, ...) value(x, "colData"))
@@ -275,6 +307,19 @@ setReplaceMethod("colData", c("SummarizedExperiment", "DataFrame"),
     x
 })
 
+setMethod("assayNames", "SummarizedExperiment",
+    function(x, ...)
+{
+    names(assays(x, withDimnames=FALSE))
+})
+
+setMethod("assayNames<-", c("SummarizedExperiment", "character"),
+    function(x, ..., value)
+{
+    names(assays(x, withDimnames=FALSE)) <- value
+    x
+})
+
 setMethod(assays, "SummarizedExperiment",
     function(x, ..., withDimnames=TRUE)
 {
@@ -285,8 +330,16 @@ setMethod(assays, "SummarizedExperiment",
 })
 
 .SummarizedExperiment.assays.replace <-
-    function(x, ..., value)
+    function(x, ..., withDimnames=TRUE, value)
 {
+    ## withDimnames arg allows names(assays(se, withDimnames=FALSE)) <- value
+    ok <- vapply(value, function(elt, xdimnames) {
+        e <- dimnames(elt)
+        (is.null(e[[1]]) || identical(e[[1]], xdimnames[[1]])) &&
+             (is.null(e[[2]]) || identical(e[[2]], xdimnames[[2]]))
+    }, logical(1), xdimnames=dimnames(x))
+    if (!all(ok))
+        stop("current and replacement dimnames() differ")
     x <- clone(x, ..., assays=value)
     msg <- .valid.SummarizedExperiment(x)
     if (!is.null(msg))
@@ -298,14 +351,9 @@ setReplaceMethod("assays", c("SummarizedExperiment", "SimpleList"),
     .SummarizedExperiment.assays.replace)
 
 setReplaceMethod("assays", c("SummarizedExperiment", "list"),
-    function(x, ..., value)
-{
-    value <- lapply(x, "dimnames<-", NULL)
-    .SummarizedExperiment.assays.replace(x, ...,
-                                         value=SimpleList(value))
-})
+    .SummarizedExperiment.assays.replace)
 
-## convenience for common use case 
+## convenience for common use case
 setMethod(assay, c("SummarizedExperiment", "missing"),
     function(x, i, ...)
 {
@@ -317,7 +365,7 @@ setMethod(assay, c("SummarizedExperiment", "missing"),
 })
 
 setMethod(assay, c("SummarizedExperiment", "numeric"),
-    function(x, i, ...) 
+    function(x, i, ...)
 {
     tryCatch({
         assays(x, ...)[[i]]
@@ -328,7 +376,7 @@ setMethod(assay, c("SummarizedExperiment", "numeric"),
 })
 
 setMethod(assay, c("SummarizedExperiment", "character"),
-    function(x, i = names(x)[1], ...) 
+    function(x, i = names(x)[1], ...)
 {
     msg <- paste0("'assay(<", class(x), ">, i=\"character\", ...)' ",
                   "invalid subscript 'i'")
@@ -374,27 +422,27 @@ setReplaceMethod("assay",
 ## <SummarizedExperiment> but not assays. dimnames need to be added on
 ## to assays when assays() invoked
 setMethod(dim, "SummarizedExperiment",
-    function(x) 
+    function(x)
 {
-    c(length(rowData(x)), nrow(colData(x)))
+    c(length(rowRanges(x)), nrow(colData(x)))
 })
 
 setMethod(dimnames, "SummarizedExperiment",
     function(x)
 {
-    list(names(rowData(x)), rownames(colData(x)))
+    list(names(rowRanges(x)), rownames(colData(x)))
 })
 
 setReplaceMethod("dimnames", c("SummarizedExperiment", "list"),
-    function(x, value) 
+    function(x, value)
 {
-    rowData <- rowData(x)
-    names(rowData) <- value[[1]]
+    rowRanges <- rowRanges(x)
+    names(rowRanges) <- value[[1]]
     colData <- colData(x)
     rownames(colData) <- value[[2]]
-    clone(x, rowData=rowData, colData=colData)
+    clone(x, rowData=rowRanges, colData=colData)
 })
- 
+
 setReplaceMethod("dimnames", c("SummarizedExperiment", "NULL"),
     function(x, value)
 {
@@ -422,24 +470,30 @@ setReplaceMethod("dimnames", c("SummarizedExperiment", "NULL"),
     ## need to expand Rle's for subsetting standard matrix
     if (!missing(i) && !missing(j)) {
         fun <- function(x) {
-            if (length(dim(x)) == 2L)
-                x[i, j, drop=FALSE]
-            else
-                x[i, j, , drop=FALSE]
+            switch(length(dim(x)),
+                   stop("'[' on assays() with 1 dimension not supported"),
+                   x[i, j, drop=FALSE],
+                   x[i, j, , drop=FALSE],
+                   x[i, j, , , drop=FALSE],
+                   stop("'[' on assays() with >4 dimensions not supported"))
         }
     } else if (!missing(i)) {
         fun <- function(x) {
-            if (length(dim(x)) == 2L)
-                x[i, , drop=FALSE]
-            else
-                x[i, , , drop=FALSE]
+            switch(length(dim(x)),
+                   stop("'[' on assays() with 1 dimension not supported"),
+                   x[i, , drop=FALSE],
+                   x[i, , , drop=FALSE],
+                   x[i, , , , drop=FALSE],
+                   stop("'[' on assays() with >4 dimensions not supported"))
         }
     } else if (!missing(j)) {
         fun <- function(x) {
-            if (length(dim(x)) == 2L)
-                x[, j, drop=FALSE]
-            else
-                x[, j, , drop=FALSE]
+            switch(length(dim(x)),
+                   stop("'[' on assays() with 1 dimension not supported"),
+                   x[, j, drop=FALSE],
+                   x[, j, , drop=FALSE],
+                   x[, j, , , drop=FALSE],
+                   stop("'[' on assays() with >4 dimensions not supported"))
         }
     }
     endoapply(assays(x, withDimnames=FALSE), fun)
@@ -466,7 +520,7 @@ setMethod("[", c("SummarizedExperiment", "ANY", "ANY"),
     if (!missing(i) && !missing(j)) {
         ii <- as.vector(i)
         jj <- as.vector(j)
-        x <- clone(x, ..., rowData=rowData(x)[i],
+        x <- clone(x, ..., rowData=rowRanges(x)[i],
             colData=colData(x)[j, , drop=FALSE],
             assays=.SummarizedExperiment.assays.subset(x, ii, jj))
     } else if (missing(i)) {
@@ -475,7 +529,7 @@ setMethod("[", c("SummarizedExperiment", "ANY", "ANY"),
             assays=.SummarizedExperiment.assays.subset(x, j=jj))
     } else {                            # missing(j)
         ii <- as.vector(i)
-        x <- clone(x, ..., rowData=rowData(x)[i],
+        x <- clone(x, ..., rowData=rowRanges(x)[i],
             assays=.SummarizedExperiment.assays.subset(x, ii))
     }
     x
@@ -487,26 +541,32 @@ setMethod("[", c("SummarizedExperiment", "ANY", "ANY"),
     ## need to expand Rle's for subsetting standard matrix
     if (!missing(i) && !missing(j)) {
         fun <- function(x, value) {
-            if (length(dim(x)) == 2L)
-                x[i, j] <- value
-            else
-                x[i, j, ] <- value
+            switch(length(dim(x)),
+                   stop("'[<-' on assays() with 1 dimension not supported"),
+                   x[i, j] <- value,
+                   x[i, j, ] <- value,
+                   x[i, j, , ] <- value,
+                   stop("'[<-' on assays() with >4 dimensions not supported"))
             x
         }
     } else if (!missing(i)) {
         fun <- function(x, value) {
-            if (length(dim(x)) == 2L)
-                x[i, ] <- value
-            else
-                x[i, , ] <- value
+            switch(length(dim(x)),
+                   stop("'[<-' on assays() with 1 dimension not supported"),
+                   x[i, ] <- value,
+                   x[i, , ] <- value,
+                   x[i, , , ] <- value,
+                   stop("'[<-' on assays() with >4 dimensions not supported"))
             x
         }
     } else if (!missing(j)) {
         fun <- function(x, value) {
-            if (length(dim(x)) == 2L)
-                x[, j] <- value
-            else
-                x[, j, ] <- value
+            switch(length(dim(x)),
+                   stop("'[<-' on assays() with 1 dimension not supported"),
+                   x[, j] <- value,
+                   x[, j, ] <- value,
+                   x[, j, , ] <- value,
+                   stop("'[<-' on assays() with >4 dimensions not supported"))
             x
         }
     }
@@ -521,7 +581,7 @@ setReplaceMethod("[",
 {
     if (missing(i) && missing(j))
         return(value)
- 
+
     if (!missing(i) && is.character(i)) {
         fmt <- paste0("<", class(x), ">[i,] index out of bounds: %s")
         i <- .SummarizedExperiment.charbound(i, rownames(x), fmt)
@@ -537,9 +597,9 @@ setReplaceMethod("[",
         jj <- as.vector(j)
         x <- clone(x, ..., exptData=c(exptData(x), exptData(value)),
             rowData=local({
-                r <- rowData(x)
-                r[i] <- rowData(value)
-                names(r)[ii] <- names(rowData(value))
+                r <- rowRanges(x)
+                r[i] <- rowRanges(value)
+                names(r)[ii] <- names(rowRanges(value))
                 r
             }), colData=local({
                 c <- colData(x)
@@ -564,13 +624,13 @@ setReplaceMethod("[",
         ii <- as.vector(i)
         x <- clone(x, ..., exptData=c(exptData(x), exptData(value)),
             rowData=local({
-                r <- rowData(x)
-                r[i] <- rowData(value)
-                names(r)[ii] <- names(rowData(value))
+                r <- rowRanges(x)
+                r[i] <- rowRanges(value)
+                names(r)[ii] <- names(rowRanges(value))
                 r
             }), assays=.SummarizedExperiment.assays.subsetgets(x, ii,
                   ..., value=value))
-        msg <- .valid.SummarizedExperiment.rowData_dims(x)
+        msg <- .valid.SummarizedExperiment.rowRanges_dims(x)
     }
     if (!is.null(msg))
         stop(msg)
@@ -593,15 +653,13 @@ setMethod("rbind", "SummarizedExperiment",
             stop("'...' objects must have the same colnames")
     if (!.compare(lapply(args, ncol)))
             stop("'...' objects must have the same number of samples")
-    rowData <- do.call(c, lapply(args, 
-        function(i) slot(i, "rowData")))
-    cdlst <- lapply(args, colData) 
-    colData <- .cbind.DataFrame(args, colData, "colData") 
+    rowRanges <- do.call(c, lapply(args, rowRanges))
+    colData <- .cbind.DataFrame(args, colData, "colData")
     assays <- .bind.arrays(args, rbind, "assays")
     exptData <- do.call(c, lapply(args, exptData))
 
-    SummarizedExperiment(assays=assays, rowData=rowData, 
-                         colData=colData, exptData=exptData) 
+    initialize(args[[1]], assays=assays, rowData=rowRanges,
+               colData=colData, exptData=exptData)
 }
 
 ## Appropriate for objects with same ranges and different samples.
@@ -614,36 +672,36 @@ setMethod("cbind", "SummarizedExperiment",
 
 .cbind.SummarizedExperiment <- function(args)
 {
-    if (!.compare(lapply(args, rowData), TRUE))
+    if (!.compare(lapply(args, rowRanges), TRUE))
         stop("'...' object ranges (rows) are not compatible")
-    rowData <- rowData(args[[1]])
-    mcols(rowData) <- .cbind.DataFrame(args, mcols, "mcols")
+    rowRanges <- rowRanges(args[[1]])
+    mcols(rowRanges) <- .cbind.DataFrame(args, mcols, "mcols")
     colData <- do.call(rbind, lapply(args, colData))
     assays <- .bind.arrays(args, cbind, "assays")
     exptData <- do.call(c, lapply(args, exptData))
 
-    SummarizedExperiment(assays=assays, rowData=rowData, 
-                         colData=colData, exptData=exptData) 
+    initialize(args[[1]], assays=assays, rowData=rowRanges,
+               colData=colData, exptData=exptData)
 }
 
-.compare <- function(x, GenomicRanges=FALSE) 
+.compare <- function(x, GenomicRanges=FALSE)
 {
     x1 <- x[[1]]
     if (GenomicRanges) {
         if (is(x1, "GRangesList")) {
-            x <- lapply(x, unlist) 
+            x <- lapply(x, unlist)
             x1 <- x[[1]]
         }
-        for (i in seq_along(x[-1])) {
+        for (i in seq_along(x)[-1]) {
             if (length(x1) != length(x[[i]]))
                 return(FALSE)
             ok <- x1 == x[[i]]
             if (!all(ok))
                 return(FALSE)
         }
-        return(TRUE) 
+        return(TRUE)
     } else {
-        all(sapply(x[-1], 
+        all(sapply(x[-1],
             function(xelt) all(identical(xelt, x[[1]]))))
     }
 }
@@ -656,10 +714,10 @@ setMethod("cbind", "SummarizedExperiment",
         nmsv <- unlist(nms, use.names=FALSE)
         names(nmsv) <- rep(seq_along(nms), elementLengths(nms))
         dups <- duplicated(nmsv)
-        ## no duplicates 
+        ## no duplicates
         if (!any(dups))
             return(do.call(cbind, lst))
-        ## confirm duplicates are the same 
+        ## confirm duplicates are the same
         lapply(nmsv[duplicated(nmsv)], function(d) {
             if (!.compare(lapply(lst, "[", d)))
                 stop("column(s) '", unname(d),
@@ -672,35 +730,43 @@ setMethod("cbind", "SummarizedExperiment",
     }
 }
 
-.bind.arrays <- function(args, bind, accessor, accessorName)
-{
-    lst <- lapply(args, accessor)
-    var <- lapply(lst,  names)
-    if (is.null(uvar <- unique(unlist(var))))
-        return(SimpleList())
-    if (!.compare(var))
-        stop("elements in ", sQuote(accessorName), 
-             " must have the same names")
-
-    ## extract variable
-    l1 <- lapply(uvar, function(v) {
-        l2 <- lapply(lst, "[[", v)
-        dim <- .assay.dimension(l2, bind)
-        if (!is.na(dim[3])) {
-            ## assay: combine each dimension of each element
-            l3 <- lapply(1:dim[3], 
-                      function(i) do.call(bind, lapply(l2, "[", ,,i)))
-            array(do.call(c, l3), dim=dim)
-        } else {
-            ## matrix: combine each element
-            do.call(bind, l2)
-        }
-    })
-    names(l1) <- uvar
-    SimpleList(l1)
+.bind.array.elements <- function(index, lst, bind) {
+    e1 <- lapply(lst, "[[", index)
+    dim <- .get.assay.dimension(e1, bind)
+    if (is.na(dim[3])) {
+        do.call(bind, e1)
+    } else {
+        e2 <- lapply(1:dim[3], function(i) {
+            do.call(bind, lapply(e1, "[", ,,i))
+        })
+        array(do.call(c, e2), dim=dim)
+    }
 }
 
-.assay.dimension <- function(lst, bind)
+.bind.arrays <- function(args, bind, accessor)
+{
+    lst <- lapply(args, accessor)
+    if (!length(elts <- unique(elementLengths(lst))) == 1L)
+        stop("elements in ", sQuote(accessor),
+             " must have the same length")
+    if (elts == 0L)
+        return(.ShallowSimpleListAssays(data=SimpleList()))
+    var <- lapply(lst,  names)
+    if (is.null(uvar <- unique(unlist(var)))) {
+        ## no names, match by position
+        res <- lapply(seq_along(elts), .bind.array.elements, lst=lst, bind=bind)
+    } else {
+        ## match by name
+        if (!.compare(var))
+            stop("elements in ", sQuote(accessor),
+                 " must have the same names")
+        res <- lapply(uvar, .bind.array.elements, lst=lst, bind=bind)
+        names(res) <- uvar
+    }
+    .ShallowSimpleListAssays(data=SimpleList(res))
+}
+
+.get.assay.dimension <- function(lst, bind)
 {
     dim <- lapply(lst, dim)
     if (!.compare(lapply(dim, "[", 3)))
@@ -727,6 +793,9 @@ setReplaceMethod("[[",
     x
 })
 
+.DollarNames.SummarizedExperiment <- function(x, pattern)
+    grep(pattern, names(colData(x)), value=TRUE)
+
 setMethod("$", "SummarizedExperiment",
     function(x, name)
 {
@@ -778,9 +847,254 @@ setMethod(show, "SummarizedExperiment",
     dlen <- sapply(dimnames, length)
     if (dlen[[1]]) scat("rownames(%d): %s\n", dimnames[[1]])
     else scat("rownames: NULL\n")
-    scat("rowData metadata column names(%d): %s\n",
-         names(mcols(rowData(object))))
+    scat("rowRanges metadata column names(%d): %s\n",
+         names(mcols(rowRanges(object))))
     if (dlen[[2]]) scat("colnames(%d): %s\n", dimnames[[2]])
     else cat("colnames: NULL\n")
     scat("colData names(%d): %s\n", names(colData(object)))
 })
+
+.from_rowRanges_to_FeatureData <- function(from)
+{
+    if (is(from, "GRanges")) {
+        fd <- .from_GRanges_to_FeatureData(from)
+    } else if (is(from, "GRangesList")) {
+        fd <- .from_GRangesList_to_FeatureData(from)
+    } else {
+        stop("class: ", sQuote(class(from)), " is not a supported type for rowRanges coercion")
+    }
+    Biobase::featureNames(fd) <- names(from)
+    fd
+}
+
+.from_GRanges_to_FeatureData <- function(from)
+{
+    data <- as.data.frame(from)
+
+    ## the first mcols are automatically included in the data.frame from
+    ## as.data.frame, the secondary mcols holds the metadata for the first
+    ## metadata columns.
+    metaData <- mcols(mcols(from))
+    if (is.null(metaData)) {
+        metaData <- as.data.frame(matrix(ncol=0, nrow=NCOL(data)))
+    } else {
+        metaData <- as.data.frame(metaData)
+    }
+    Biobase::AnnotatedDataFrame(data, metaData)
+}
+.from_GRangesList_to_FeatureData <- function(from)
+{
+    data <- as.data.frame(mcols(from))
+
+    ## the first mcols are automatically included in the data.frame from
+    ## as.data.frame, the secondary mcols holds the metadata for the first
+    ## metadata columns.
+    metaData <- mcols(mcols(from))
+    if (is.null(metaData)) {
+        metaData <- as.data.frame(matrix(ncol=0, nrow=NCOL(data)))
+    } else {
+        metaData <- as.data.frame(metaData)
+    }
+    Biobase::AnnotatedDataFrame(data, metaData)
+}
+
+.from_AnnotatedDataFrame_to_DataFrame <- function(from)
+{
+    df <- DataFrame(Biobase::pData(from))
+    mcols(df) <- DataFrame(Biobase::varMetadata(from))
+    df
+}
+
+.from_DataFrame_to_AnnotatedDataFrame <- function(df)
+{
+    data <- as(df, "data.frame")
+    metaData <- mcols(df)
+    if (is.null(metaData)) {
+        metaData <- as.data.frame(matrix(ncol=0, nrow=NCOL(data)))
+    } else {
+        metaData <- as(metaData, "data.frame")
+    }
+    Biobase::AnnotatedDataFrame(data, metaData)
+}
+
+# If the ExpressionSet has featureData with range information make GRanges out
+# of that, otherwise make an empty GRangesList with names from the featureNames
+naiveRangeMapper <- function(from)
+{
+    nms <- Biobase::featureNames(from)
+    res <- tryCatch(makeGRangesFromDataFrame(Biobase::pData(Biobase::featureData(from)), keep.extra.columns = TRUE),
+             error = function(e) {
+                 res <- relist(GRanges(), vector("list", length=length(nms)))
+                 mcols(res) <- .from_AnnotatedDataFrame_to_DataFrame(Biobase::featureData(from))
+                 res
+             })
+    names(res) <- nms
+    res
+}
+
+# Simple ProbeId to Range mapper
+# Probes with multiple ranges are dropped
+# The sign of the chromosome location is assumed to contain the strand
+# information
+probeRangeMapper <- function(from)
+{
+    annotation <- Biobase::annotation(from)
+    if (identical(annotation, character(0))) {
+        return(naiveRangeMapper(from))
+    }
+    if (requireNamespace("annotate", quietly = TRUE)) {
+        annotationPackage <- annotate::annPkgName(annotation)
+        if (require(annotationPackage, character.only = TRUE, quietly = TRUE)) {
+            db <- get(annotationPackage, envir = asNamespace(annotationPackage))
+            pid <- Biobase::featureNames(from)
+            locs <- AnnotationDbi::select(db, pid, columns = c("CHR", "CHRLOC", "CHRLOCEND"))
+            locs <- na.omit(locs)
+            dups <- duplicated(locs$PROBEID)
+            if (any(dups)) {
+                locs <- locs[!dups, , drop = FALSE]
+            }
+            strand <- ifelse(locs$CHRLOC > 0, "+", "-")
+            res <- GRanges(seqnames = locs$CHR,
+                           ranges = IRanges(abs(locs$CHRLOC), abs(locs$CHRLOCEND)),
+                           strand = strand)
+            names(res) <- locs$PROBEID
+
+            if (NROW(res) < length(pid)) {
+                warning(length(pid) - NROW(res), " probes could not be mapped.", call. = FALSE)
+            }
+            res
+        } else {
+            stop("Failed to load ", sQuote(annotationPackage), " package", call. = FALSE)
+        }
+    } else {
+        stop("Failed to load annotate package", call. = FALSE)
+    }
+}
+
+# Simple ProbeId to Gene mapper
+# Is there a way to get the txDb given the annotation package?
+geneRangeMapper <- function(txDbPackage, key = "ENTREZID")
+{
+    function(from) {
+        annotation <- Biobase::annotation(from)
+        if (identical(annotation, character(0))) {
+            return(naiveRangeMapper(from))
+        }
+        if (requireNamespace("annotate", quietly = TRUE)) {
+            annotationPackage <- annotate::annPkgName(annotation)
+            if (require(annotationPackage, character.only = TRUE, quietly = TRUE)) {
+                db <- get(annotationPackage, envir = asNamespace(annotationPackage))
+                pid <- Biobase::featureNames(from)
+                probeIdToGeneId <- AnnotationDbi::mapIds(db, pid, key, "PROBEID")
+                geneIdToProbeId <- setNames(names(probeIdToGeneId), probeIdToGeneId)
+
+                if (requireNamespace(txDbPackage, quietly = TRUE)) {
+                    txDb <- get(txDbPackage, envir = asNamespace(txDbPackage))
+                    genes <- GenomicFeatures::genes(txDb)
+                    probesWithAMatch <- probeIdToGeneId[probeIdToGeneId %in% names(genes)]
+                    res <- genes[probesWithAMatch]
+                    names(res) <- geneIdToProbeId[names(res)]
+                    if (NROW(res) < length(pid)) {
+                        warning(length(pid) - NROW(res), " probes could not be mapped.", call. = FALSE)
+                    }
+                    res
+                } else {
+                    stop("Failed to load ", sQuote(txDbPackage), " package", call. = FALSE)
+                }
+
+            } else {
+                stop("Failed to load ", sQuote(annotationPackage), " package", call. = FALSE)
+            }
+        } else {
+            stop("Failed to load annotate package", call. = FALSE)
+        }
+    }
+}
+
+makeSummarizedExperimentFromExpressionSet <- function(from, mapFun = naiveRangeMapper)
+{
+    if (requireNamespace("Biobase", quietly = TRUE)) {
+        mapFun <- match.fun(mapFun)
+        rowRanges <- mapFun(from)
+        matches <- match(names(rowRanges),
+                         Biobase::featureNames(from),
+                         nomatch = 0)
+        from <- from[matches, drop = FALSE]
+        assays <- as.list(Biobase::assayData(from))
+        colData <- .from_AnnotatedDataFrame_to_DataFrame(Biobase::phenoData(from))
+        exptData <- SimpleList(
+                               experimentData = Biobase::experimentData(from),
+                               annotation = Biobase::annotation(from),
+                               protocolData = Biobase::protocolData(from)
+                               )
+
+        SummarizedExperiment(
+                             assays = assays,
+                             rowRanges = rowRanges,
+                             colData = colData,
+                             exptData = exptData
+                             )
+    }
+}
+
+suppressMessages(
+    setAs("ExpressionSet", "SummarizedExperiment", function(from)
+    {
+        makeSummarizedExperimentFromExpressionSet(from, naiveRangeMapper)
+    })
+)
+
+suppressMessages(
+    setAs("SummarizedExperiment", "ExpressionSet", function(from)
+    {
+        if (requireNamespace("Biobase", quietly = TRUE)) {
+
+            assayData <- list2env(as.list(assays(from)))
+
+            numAssays <- length(assayData)
+
+            if(numAssays == 0) {
+                assayData$exprs <- new("matrix")
+            } else if (!"exprs" %in% ls(assayData)) {
+                ## if there isn't an exprs assay we need to pick one as exprs, so
+                ## rename the first element exprs and issue a warning.
+                exprs <- ls(assayData)[[1]]
+                warning("No assay named ", sQuote("exprs"), " found, renaming ",
+                        exprs, " to ", sQuote("exprs"), ".")
+                assayData[["exprs"]] <- assayData[[exprs]]
+                rm(list=exprs, envir=assayData)
+            }
+
+            featureData <- .from_rowRanges_to_FeatureData(rowRanges(from))
+            phenoData <- .from_DataFrame_to_AnnotatedDataFrame(colData(from))
+
+            ed <- exptData(from)
+
+            experimentData <- if (!is.null(ed$experimentData)) {
+                ed$experimentData
+            } else {
+                Biobase::MIAME()
+            }
+
+            annotation <- if (!is.null(ed$annotation)) {
+                ed$annotation
+            } else {
+                character()
+            }
+
+            protocolData <- if (!is.null(ed$protocolData)) {
+                ed$protocolData
+            } else {
+                Biobase::annotatedDataFrameFrom(assayData, byrow=FALSE)
+            }
+
+            Biobase::ExpressionSet(assayData,
+                                   phenoData = phenoData,
+                                   featureData = featureData,
+                                   experimentData = experimentData,
+                                   annotation = annotation,
+                                   protocolData = protocolData
+                                   )
+        }
+    })
+)
