@@ -42,17 +42,15 @@
 }
 
 .interIntervalGenomicRanges2 <- function(x, FUN,
-                                         with.revmap=FALSE, with.mapping=FALSE,
+                                         with.revmap=FALSE,
                                          ignore.strand=FALSE, ...)
 {
     x <- clone(x)
     mcols(x) <- NULL
     f <- .interIntervalSplitVariable(x, ignore.strand=ignore.strand)
     xIRangesList <- split(unname(ranges(x)), f)
-    if (with.revmap || with.mapping) {
-        ansIRangesList <- FUN(xIRangesList,
-                              with.revmap=with.revmap,
-                              with.mapping=with.mapping, ...)
+    if (with.revmap) {
+        ansIRangesList <- FUN(xIRangesList, with.revmap=with.revmap, ...)
     } else {
         ansIRangesList <- FUN(xIRangesList, ...)
     }
@@ -74,23 +72,18 @@
         ans_mcols <- new("DataFrame", nrows=length(ans_seqnames))
     } else {
         mcols(ans_ranges) <- NULL
-        if (with.revmap || with.mapping) {
-            if (with.revmap) {
-                revmap_colname <- "revmap"
-            } else {
-                revmap_colname <- "mapping"
-            }
-            revmap_unlisted <- ans_mcols[[revmap_colname]]
+        if (with.revmap) {
+            revmap_unlisted <- ans_mcols[["revmap"]]
             revmap_partitioning <- PartitioningByEnd(ansIRangesList)
             old2new <- splitAsList(seq_along(x), f)
             revmap_unlisted2 <- .translateRevmap(revmap_unlisted,
                                                  revmap_partitioning,
                                                  old2new)
-            ans_mcols[[revmap_colname]] <- revmap_unlisted2
+            ans_mcols[["revmap"]] <- revmap_unlisted2
         }
     }
     if (length(extraColumnSlotNames(x)) > 0L)
-        x <- new("GRanges", x)
+        x <- granges(x)
     update(x, seqnames=ans_seqnames,
               ranges=ans_ranges,
               strand=ans_strand,
@@ -113,7 +106,7 @@
     ansSeqnames <- Rle(factor(listNameMatrix[1L, ], levels=seqlevels(x)), k)
     ansStrand <- Rle(strand(listNameMatrix[2L, ]), k)
     if (length(extraColumnSlotNames(x)) > 0L)
-        x <- new("GRanges", x)
+        x <- granges(x)
     update(x, seqnames=ansSeqnames,
            ranges=unlist(ansIRangesList, use.names=FALSE),
            strand=ansStrand,
@@ -152,7 +145,7 @@ setMethod("range", "GRangesList",
 
 setMethod("reduce", "GenomicRanges",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE, with.mapping=FALSE,
+                with.revmap=FALSE,
                 with.inframe.attrib=FALSE, ignore.strand=FALSE)
     {
         if (!identical(with.inframe.attrib, FALSE))
@@ -165,8 +158,7 @@ setMethod("reduce", "GenomicRanges",
         .interIntervalGenomicRanges2(x, reduce,
                                      drop.empty.ranges=drop.empty.ranges,
                                      min.gapwidth=min.gapwidth,
-                                     with.revmap=with.revmap,
-                                     with.mapping=with.mapping)
+                                     with.revmap=with.revmap)
     }
 )
 
@@ -199,9 +191,9 @@ setMethod("gaps", "GenomicRanges",
             start <- start[seqlevels]
         if (!is.null(names(end)))
             end <- end[seqlevels]
-        start <- IRanges:::recycleVector(start, length(seqlevels))
+        start <- S4Vectors:::recycleVector(start, length(seqlevels))
         start <- rep(start, each=3L)
-        end <- IRanges:::recycleVector(end, length(seqlevels))
+        end <- S4Vectors:::recycleVector(end, length(seqlevels))
         end <- rep(end, each=3L)
         .interIntervalGenomicRanges(x, gaps, start=start, end=end)
     }
@@ -217,7 +209,21 @@ setMethod("disjoin", "GenomicRanges",
         .interIntervalGenomicRanges2(x, disjoin, ignore.strand=ignore.strand)
 )
 
-applyOnRangesBySpace <- function(x, FUN, ..., ignore.strand = FALSE) {
+setMethod("disjoin", "GRangesList",
+    function(x, ignore.strand=FALSE)
+    {
+        gr <- deconstructGRLintoGR(x)
+        d <- disjoin(gr, ignore.strand=ignore.strand)
+        reconstructGRLfromGR(d, x)
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### isDisjoint()
+###
+
+.applyOnRangesBySpace <- function(x, FUN, ..., ignore.strand = FALSE) {
   if (ignore.strand)
     f <- seqnames(x)
   else
@@ -229,23 +235,10 @@ applyOnRangesBySpace <- function(x, FUN, ..., ignore.strand = FALSE) {
   ans
 }
 
-setMethod("disjoin", "GRangesList",
-    function(x, ...)
-{
-    gr <- deconstructGRLintoGR(x)
-    d <- disjoin(gr, ...)
-    reconstructGRLfromGR(d, x)
-})
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### isDisjoint()
-###
-
 setMethod("isDisjoint", "GenomicRanges",
     function(x, ignore.strand=FALSE)
     {
-        all(applyOnRangesBySpace(x, isDisjoint, ignore.strand = ignore.strand))
+        all(.applyOnRangesBySpace(x, isDisjoint, ignore.strand = ignore.strand))
     }
 )
 
@@ -282,7 +275,7 @@ setMethod("isDisjoint", "GRangesList",
 
 setMethod("disjointBins", "GenomicRanges",
           function(x, ignore.strand = FALSE) {
-            applyOnRangesBySpace(x, disjointBins,
-                                 ignore.strand = ignore.strand)
+            .applyOnRangesBySpace(x, disjointBins,
+                                  ignore.strand = ignore.strand)
           })
 
