@@ -14,24 +14,26 @@
 setMethod("shift", "GenomicRanges",
     function(x, shift=0L, use.names=TRUE)
     {
-        ranges <- shift(ranges(x), shift, use.names=use.names)
-        clone(x, ranges=ranges) 
+        new_ranges <- shift(ranges(x), shift=shift, use.names=use.names)
+        clone(x, ranges=new_ranges) 
     }
 )
 
 setMethod("shift", "GRangesList",
     function(x, shift=0L, use.names=TRUE)
     {
-        if (is(shift, "IntegerList")) {
-            if (length(shift) != length(x) ||
-                any(elementLengths(shift) != elementLengths(x))) {
-                stop("IntegerList 'shift' not of same dimension as 'x'")
-            }
-            shift <- unlist(shift, use.names=FALSE)
-        }
-        ranges(x@unlistData) <-
-          shift(x@unlistData@ranges, shift, use.names=use.names)
-        x
+        ## Unlist 'x'.
+        unlisted_x <- unlist(x, use.names=FALSE)
+        ## Recycle and unlist 'shift'.
+        if (!is(shift, "List"))
+            shift <- as(shift, "List")
+        shift <- S4Vectors:::VH_recycle(shift, x, "shift", "x")
+        unlisted_shift <- unlist(shift, use.names=FALSE)
+        ## Compute unlisted 'ans'.
+        unlisted_ans <- shift(unlisted_x, shift=unlisted_shift,
+                              use.names=use.names)
+        ## Relist and return.
+        relist(unlisted_ans, x)
     }
 )
 
@@ -43,10 +45,54 @@ setMethod("shift", "GRangesList",
 setMethod("narrow", "GenomicRanges",
     function(x, start=NA, end=NA, width=NA, use.names=TRUE)
     {
-        rng <- narrow(ranges(x), start=start, end=end, width=width,
-                      use.names=TRUE)
-        ranges(x) <- rng
-        x
+        new_ranges <- narrow(ranges(x), start=start, end=end, width=width,
+                                        use.names=use.names)
+        clone(x, ranges=new_ranges) 
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### resize()
+###
+
+setMethod("resize", "GenomicRanges",
+    function(x, width, fix="start", use.names=TRUE, ignore.strand=FALSE)
+    {
+        if (!missing(fix) && length(x) > 0L && 
+            (length(fix) > length(x) || length(x) %% length(fix) > 0L))
+            stop("'x' is not a multiple of 'fix' length")
+        if (!isTRUEorFALSE(ignore.strand))
+            stop("'ignore.strand' must be TRUE or FALSE")
+        if (ignore.strand) {
+            fix <- Rle(rep.int(fix, length(x)))
+        } else {
+            revFix <- c(start="end", end="start", center="center")
+            if (length(x) == 0L)
+              fix <- character()
+            else fix <- ifelse(strand(x) == "-", revFix[fix], fix)
+        }
+        new_ranges <- resize(ranges(x), width=width, fix=fix,
+                                        use.names=use.names)
+        clone(x, ranges=new_ranges)
+    }
+)
+
+setMethod("resize", "GRangesList",
+    function(x, width, fix="start", use.names=TRUE, ignore.strand=FALSE)
+    {
+        ## Unlist 'x'.
+        unlisted_x <- unlist(x, use.names=FALSE)
+        ## Recycle and unlist 'width'.
+        if (!is(width, "List"))
+            width <- as(width, "List")
+        width <- S4Vectors:::VH_recycle(width, x, "width", "x")
+        unlisted_width <- unlist(width, use.names=FALSE)
+        ## Compute unlisted 'ans'.
+        unlisted_ans <- resize(unlisted_x, unlisted_width, fix=fix, 
+                               use.names=use.names, ignore.strand=ignore.strand)
+        ## Relist and return.
+        relist(unlisted_ans, x)
     }
 )
 
@@ -65,16 +111,9 @@ setMethod("flank", "GenomicRanges",
             start <- rep.int(start, length(x))
         else 
             start <- as.vector(start == (strand(x) != "-"))
-        ranges <-
-            flank(ranges(x), width=width, start=start, both=both,
-                  use.names=use.names)
-        if (!IRanges:::anyMissing(seqlengths(x))) {
-            start(x) <- start(ranges)
-            end(x) <- end(ranges)
-        } else {
-            x <- clone(x, ranges=ranges)
-        }
-        x
+        new_ranges <- flank(ranges(x), width=width, start=start, both=both,
+                                       use.names=use.names)
+        clone(x, ranges=new_ranges) 
     }
 )
 
@@ -82,11 +121,21 @@ setMethod("flank", "GRangesList",
     function(x, width, start=TRUE, both=FALSE, use.names=TRUE,
              ignore.strand=FALSE)
     {
-        x@unlistData <- flank(x@unlistData, width = width, start = start,
-                              both = both, use.names = use.names,
-                              ignore.strand = ignore.strand)
-        x
-    })
+        ## Unlist 'x'.
+        unlisted_x <- unlist(x, use.names=FALSE)
+        ## Recycle and unlist 'width'.
+        if (!is(width, "List"))
+            width <- as(width, "List")
+        width <- S4Vectors:::VH_recycle(width, x, "width", "x")
+        unlisted_width <- unlist(width, use.names=FALSE)
+        ## Compute unlisted 'ans'.
+        unlisted_ans <- flank(unlisted_x, unlisted_width,
+                              start=start, both=both,
+                              use.names=use.names, ignore.strand=ignore.strand)
+        ## Relist and return.
+        relist(unlisted_ans, x)
+    }
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -137,39 +186,6 @@ setMethod("promoters", "GRangesList",
 ### TODO: Add "reflect" method for GenomicRanges objects.
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### resize()
-###
-
-setMethod("resize", "GenomicRanges",
-    function(x, width, fix="start", use.names=TRUE, ignore.strand=FALSE)
-    {
-        if (!missing(fix) && length(x) > 0L && 
-            (length(fix) > length(x) || length(x) %% length(fix) > 0L))
-            stop("'x' is not a multiple of 'fix' length")
-        if (!isTRUEorFALSE(ignore.strand))
-            stop("'ignore.strand' must be TRUE or FALSE")
-        if (ignore.strand) {
-            fix <- Rle(rep.int(fix, length(x)))
-        } else {
-            revFix <- c(start="end", end="start", center="center")
-            if (length(x) == 0L)
-              fix <- character()
-            else fix <- ifelse(strand(x) == "-", revFix[fix], fix)
-        }
-        ranges <-
-            resize(ranges(x), width=width, fix=fix, use.names=use.names)
-        if (!IRanges:::anyMissing(seqlengths(x))) {
-            start(x) <- start(ranges)
-            end(x) <- end(ranges)
-        } else {
-            x <- clone(x, ranges=ranges)
-        }
-        x
-    }
-)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### restrict()
 ###
@@ -244,16 +260,18 @@ setMethod("restrict", "GRangesList",
 setMethod("trim", "GenomicRanges",
     function(x, use.names=TRUE)
     {
-        end <- NA_integer_ 
-        if (any(!is.na(seqlengths(x)))) {
-            seqlen <- seqlengths(x)
-            seqlen[isCircular(x) %in% TRUE] <- NA_integer_
-            idx <- match(as.character(seqnames(x)), names(seqlen))
-            end <- seqlen[idx]
-        }
-        x@ranges <- restrict(ranges(x), start=1L, end=end, 
-                             keep.all.ranges=TRUE, use.names=use.names)
-        x
+        ## We trim only out-of-bound ranges located on non-circular sequences
+        ## whose length is not NA.
+        ## See get_out_of_bound_index() in GenomicRanges-class.R
+        idx <- get_out_of_bound_index(x)
+        if (length(idx) == 0L)
+            return(x)
+        new_ranges <- ranges(x)
+        seqnames_id <- as.integer(seqnames(x))[idx]
+        new_end <- unname(seqlengths(x))[seqnames_id]
+        new_ranges[idx] <- restrict(new_ranges[idx], start=1L, end=new_end,
+                                                     keep.all.ranges=TRUE)
+        clone(x, ranges=new_ranges)
     }
 )
 
@@ -265,7 +283,7 @@ setMethod("trim", "GenomicRanges",
 setMethod("Ops", c("GenomicRanges", "numeric"),
     function(e1, e2)
     {
-        if (IRanges:::anyMissing(e2))
+        if (S4Vectors:::anyMissing(e2))
             stop("NA not allowed as zoom factor")
         e2 <- recycleNumericArg(e2, "e2", length(e1))
         if (.Generic == "*") {
