@@ -29,7 +29,7 @@
     i <- findInterval(query - !leftOf, subject) + leftOf
     i[subject[i] %in% sentinel] <- NA_integer_
  
-    IRanges:::.vectorToHits(i, rle, ord)
+    IRanges:::vectorToHits(i, rle, ord)
 }
 
 .findNearest_distance <-
@@ -42,14 +42,12 @@
         start(query)[queryHits(hit)] - end(subject)[subjectHits(hit)]
 }
 
-.Hits <-
-    function(queryHits, subjectHits,
-             queryLength=as.integer(max(c(0, queryHits))),
-             subjectLength=as.integer(max(c(0, subjectHits))))
+.Hits <- function(queryHits, subjectHits,
+                  queryLength=as.integer(max(c(0, queryHits))),
+                  subjectLength=as.integer(max(c(0, subjectHits))))
 {
-    o <- IRanges:::orderIntegerPairs(queryHits, subjectHits)
-    new("Hits", queryHits=queryHits[o], subjectHits=subjectHits[o],
-        queryLength=queryLength, subjectLength=subjectLength)
+    o <- S4Vectors:::orderIntegerPairs(queryHits, subjectHits)
+    Hits(queryHits[o], subjectHits[o], queryLength, subjectLength)
 }
 
 .findPrecedeFollow_pmin <-
@@ -78,14 +76,17 @@
     i1 <- dist1 == dMin[i1]
 
     .Hits(c(queryHits(hit0)[i0], queryHits(hit1)[i1]),
-        c(subjectHits(hit0)[i0], subjectHits(hit1)[i1]),
-        queryLength(hit0), subjectLength(hit0))
+          c(subjectHits(hit0)[i0], subjectHits(hit1)[i1]),
+          queryLength(hit0), subjectLength(hit0))
 }
 
 .GenomicRanges_findPrecedeFollow <-
     function(query, subject, select, ignore.strand, 
              where=c("precede", "follow"))
 {
+    if (!length(query) || !length(subject))
+        return(Hits(queryLength=length(query), subjectLength=length(subject)))
+
     leftOf <- "precede" == match.arg(where)
     if (ignore.strand)
         strand(query) <- strand(subject) <- "+"
@@ -172,39 +173,38 @@
 ###
 
 setMethod("precede", c("GenomicRanges", "GenomicRanges"),
-    function(x, subject, select = c("arbitrary", "all"), 
-             ignore.strand=FALSE, ...)
+    function(x, subject, select=c("arbitrary", "all"), ignore.strand=FALSE)
     {
         select <- match.arg(select)
         .GenomicRanges_findPrecedeFollow(x, subject, select, ignore.strand, 
-            "precede", ...) 
+                                         "precede") 
     }
 )
 
 setMethod("precede", c("GenomicRanges", "missing"),
-    function(x, subject, select = c("arbitrary", "all"), 
-             ignore.strand=FALSE, ...)
+    function(x, subject, select=c("arbitrary", "all"), ignore.strand=FALSE)
     {
-        callGeneric(x, subject=x, select=select, ignore.strand=ignore.strand, 
-            ...) 
+        select <- match.arg(select)
+        .GenomicRanges_findPrecedeFollow(x, subject, select, ignore.strand, 
+                                         "precede") 
     }
 )
 
 setMethod("follow", c("GenomicRanges", "GenomicRanges"),
-    function(x, subject, select = c("arbitrary", "all"), 
-             ignore.strand=FALSE, ...)
+    function(x, subject, select=c("arbitrary", "all"), ignore.strand=FALSE)
     {
         select <- match.arg(select)
         .GenomicRanges_findPrecedeFollow(x, subject, select, ignore.strand, 
-            "follow", ...) 
+                                         "follow") 
     }
 )
 
 setMethod("follow", c("GenomicRanges", "missing"),
-    function(x, subject, select=c("arbitrary", "all"), ignore.strand=FALSE, ...)
+    function(x, subject, select=c("arbitrary", "all"), ignore.strand=FALSE)
     {
-        callGeneric(x, subject=x, select=select, ignore.strand=ignore.strand, 
-            ...) 
+        select <- match.arg(select)
+        .GenomicRanges_findPrecedeFollow(x, subject, select, ignore.strand, 
+                                         "follow") 
     }
 )
 
@@ -213,32 +213,27 @@ setMethod("follow", c("GenomicRanges", "missing"),
 ### nearest()
 ###
 
-.filterMatchMatrix <- function(m, i, map) 
+.filterHits <- function(hits, i, map) 
 {
-    qrle <- Rle(m[, 1L])
-    qstart <- qend <- integer(length(i))
-    qstart[runValue(qrle)] <- start(qrle)
-    qend[runValue(qrle)] <- end(qrle)
-    rows <- as.integer(IRanges(qstart[i], qend[i]))
-    m <- m[rows, , drop = FALSE]
+    m <- as.matrix(hits[as(hits, "IRanges")[i]])
     m[, 1L] <- map[m[, 1L]]
     m
 }
 
-.nearest <- function(x, subject, select, ignore.strand, ignoreSelf=FALSE, ...)
+.nearest <- function(x, subject, select, algorithm,
+                     ignore.strand, ignoreSelf=FALSE)
 {
     ## overlapping ranges
     if (ignoreSelf) {
-        fo <- findOverlaps(x, subject, select="all",
-                           ignore.strand=ignore.strand)
-        ol <- IRanges:::processSelfMatching(fo, select, ignoreSelf=TRUE)
+        ol <- findOverlaps(x, select=select, algorithm=algorithm,
+                           ignore.strand=ignore.strand, ignoreSelf=TRUE)
     } else {
-        ol <- findOverlaps(x, subject, select=select,
+        ol <- findOverlaps(x, subject, select=select, algorithm=algorithm,
                            ignore.strand=ignore.strand)
     }
+
     if (select == "all") {
-        m <- as.matrix(ol)
-        olv <- IRanges:::.hitsMatrixToVector(m, length(x))
+        olv <- selectHits(ol, select="first")
     } else {
         olv <- ol
     }
@@ -248,11 +243,22 @@ setMethod("follow", c("GenomicRanges", "missing"),
         p <- precede(x, subject, select, ignore.strand)
         f <- follow(x, subject, select, ignore.strand)
 
+        ## terminate if no results
+        if (!length(p) && !length(f)) {
+            if (is(olv, "Hits") && !length(olv) || all(is.na(olv))) {
+                if (select == "all") 
+                    return(Hits(queryLength=length(x),
+                                subjectLength=length(subject)))
+                else if (select == "arbitrary")
+                    return (rep(NA, length(x)))
+            }
+        }
+
         if (select == "all") {
-            pmat <- as.matrix(p)
-            fmat <- as.matrix(f)
-            p <- IRanges:::.hitsMatrixToVector(as.matrix(pmat), length(x))
-            f <- IRanges:::.hitsMatrixToVector(as.matrix(fmat), length(x))
+            p0 <- p
+            p <- selectHits(p, select="first")
+            f0 <- f
+            f <- selectHits(f, select="first")
         }
 
         ## choose nearest or not missing
@@ -265,10 +271,9 @@ setMethod("follow", c("GenomicRanges", "missing"),
         if (select == "all") {
             map <- which(is.na(olv))
             pnearest[pdist == fdist] <- TRUE
-            m <- rbind(m, .filterMatchMatrix(pmat, pnearest, map), 
-                .filterMatchMatrix(fmat, !pnearest, map))
-            m <- m[IRanges:::orderIntegerPairs(m[, 1L], m[, 2L]), , 
-                drop = FALSE]
+            m <- rbind(as.matrix(ol), .filterHits(p0, pnearest, map),
+                                      .filterHits(f0, !pnearest, map))
+            m <- m[S4Vectors:::orderIntegerPairs(m[, 1L], m[, 2L]),, drop=FALSE]
             ol@queryHits <- unname(m[, 1L])
             ol@subjectHits <- unname(m[, 2L])
         } else {
@@ -281,27 +286,31 @@ setMethod("follow", c("GenomicRanges", "missing"),
 
 .nearestDistance <- function(x, subject, index)
 {
-    maxStart <- pmax.int(start(x), start(subject)[index])
-    minEnd <- pmin.int(end(x), end(subject)[index])
-    pmax.int(maxStart - minEnd - 1L, 0L)
+    if (length(index)) {
+        maxStart <- pmax.int(start(x), start(subject)[index])
+        minEnd <- pmin.int(end(x), end(subject)[index])
+        pmax.int(maxStart - minEnd - 1L, 0L)
+    } else NA
 }
 
 
 setMethod("nearest", c("GenomicRanges", "GenomicRanges"),
-    function(x, subject, select=c("arbitrary", "all"), ignore.strand=FALSE, 
-             ...)
+    function(x, subject, select=c("arbitrary", "all"),
+             algorithm=c("nclist", "intervaltree"), ignore.strand=FALSE)
     {
         select <- match.arg(select)
-        .nearest(x, subject, select=select, ignore.strand=ignore.strand, ...)
+        .nearest(x, subject, select=select, algorithm=match.arg(algorithm),
+                 ignore.strand=ignore.strand)
     }
 )
 
 setMethod("nearest", c("GenomicRanges", "missing"),
-    function(x, subject, select=c("arbitrary", "all"), ignore.strand=FALSE, 
-             ...)
+    function(x, subject, select=c("arbitrary", "all"),
+             algorithm=c("nclist", "intervaltree"), ignore.strand=FALSE)
     {
-        callGeneric(x, subject=x, select=select, ignore.strand=ignore.strand, 
-                    ignoreSelf=TRUE, ...)
+        select <- match.arg(select)
+        .nearest(x, x, select=select, algorithm=match.arg(algorithm),
+                 ignore.strand=ignore.strand, ignoreSelf=TRUE)
     }
 )
 
@@ -333,37 +342,45 @@ setMethod("distance", c("GenomicRanges", "GenomicRanges"),
 ###
 
 setMethod("distanceToNearest", c("GenomicRanges", "GenomicRanges"),
-    function(x, subject, ignore.strand=FALSE, ...)
+    function(x, subject, algorithm=c("nclist", "intervaltree"),
+             ignore.strand=FALSE, ...)
     {
-        x_nearest <- nearest(x, subject, ignore.strand=ignore.strand)
+        x_nearest <- nearest(x, subject, algorithm=match.arg(algorithm),
+                             ignore.strand=ignore.strand, ...)
         .distanceToNearest(x_nearest, x, subject, ignore.strand=ignore.strand)
     }
 )
 
 setMethod("distanceToNearest", c("GenomicRanges", "missing"),
-    function(x, subject, ignore.strand=FALSE, ...)
+    function(x, subject, algorithm=c("nclist", "intervaltree"),
+             ignore.strand=FALSE, ...)
     {
-        x_nearest <- nearest(x, ignore.strand=ignore.strand)
+        x_nearest <- nearest(x, algorithm=match.arg(algorithm),
+                             ignore.strand=ignore.strand, ...)
         .distanceToNearest(x_nearest, x, x, ignore.strand=ignore.strand)
     }
 )
 
 .distanceToNearest <- function(x_nearest, x, subject, ignore.strand)
-{ 
-    if (identical(integer(0), x_nearest)) {
-        DataFrame(queryHits=integer(), subjectHits=integer(),
-                  distance=integer())
+{
+    ## 'x_nearest' is Hits when select = all
+    if (is(x_nearest, "Hits")) {
+        queryHits <- queryHits(x_nearest)
+        subjectHits <- subjectHits(x_nearest)
     } else {
-        idx <- !is.na(x_nearest) 
-        queryHits=seq_len(length(x_nearest))
-        subjectHits=x_nearest
-        distance <- rep(NA_integer_, length(x_nearest))
-        distance[idx]=distance(x[idx], subject[na.omit(x_nearest)],
-                               ignore.strand=ignore.strand)
-        new("Hits", queryHits=queryHits, subjectHits=subjectHits,
-                    queryLength=length(x), 
-                    subjectLength=length(subject),
-                    elementMetadata=DataFrame(distance=distance))
+    ## 'x_nearest' is Integer vector when select = arbitrary
+        queryHits <- seq_along(x)[!is.na(x_nearest)]
+        subjectHits <- x_nearest[!is.na(x_nearest)]
+    }
+
+    if (!length(subjectHits) || all(is.na(subjectHits))) {
+        Hits(queryLength=length(x), 
+             subjectLength=length(subject),
+             distance=integer(0))
+    } else {
+        distance <- distance(x[queryHits], subject[subjectHits],
+                             ignore.strand=ignore.strand)
+        Hits(queryHits, subjectHits, length(x), length(subject), distance)
     }
 }
 
