@@ -4,6 +4,57 @@
 
 ### TODO: What's the impact of circularity on the set operations?
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### 2 low-level helper functions.
+###
+### Both return a named integer vector where the names are guaranteed to be
+### 'seqlevels(x)'.
+###
+
+.minStartPerGRangesSequence <- function(x)
+{
+    cil <- splitAsList(start(x), seqnames(x))  # CompressedIntegerList object
+    ## The 4 lines below are equivalent to:
+    ##   ans <- min(cil)
+    ##   ans[elementLengths(v) == 0L] <- NA_integer_
+    ## but much faster!
+    ## TODO: Replace with the above, but only when the "min" method for
+    ## CompressedIntegerList objects (implemented in the IRanges package)
+    ## is as fast as the "viewMins" method for XIntegerViews objects
+    ## (implemented in C in the XVector package). Ideally, the 2 methods
+    ## should share the same underlying code.
+    v <- Views(cil@unlistData, cil@partitioning)  # XIntegerViews object
+    ans <- viewMins(v)
+    ans[width(v) == 0L] <- NA_integer_
+    names(ans) <- names(v)
+    ans
+}
+
+.maxEndPerGRangesSequence <- function(x)
+{
+    cil <- splitAsList(end(x), seqnames(x))  # CompressedIntegerList object
+    ## The 4 lines below are equivalent to:
+    ##   ans <- max(cil)
+    ##   ans[elementLengths(v) == 0L] <- NA_integer_
+    ## but much faster!
+    ## TODO: Replace with the above, but only when the "max" method for
+    ## CompressedIntegerList objects (implemented in the IRanges package)
+    ## is as fast as the "viewMaxs" method for XIntegerViews objects
+    ## (implemented in C in the XVector package). Ideally, the 2 methods
+    ## should share the same underlying code.
+    v <- Views(cil@unlistData, cil@partitioning)  # XIntegerViews object
+    ans <- viewMaxs(v)
+    ans[width(v) == 0L] <- NA_integer_
+    names(ans) <- names(v)
+    ans
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### union(), intersect(), setdiff()
+###
+
 setMethod("union", c("GRanges", "GRanges"),
     function(x, y, ignore.strand=FALSE, ...)
     {
@@ -32,7 +83,7 @@ setMethod("intersect", c("GRanges", "GRanges"),
         ## If the length of a sequence is unknown (NA), then we use
         ## the max end value found on that sequence in 'x' or 'y'.
         seqlengths[is.na(seqlengths)] <-
-            maxEndPerGRangesSequence(c(x, y))[is.na(seqlengths)]
+            .maxEndPerGRangesSequence(c(x, y))[is.na(seqlengths)]
         setdiff(x, gaps(y, end=seqlengths))
     }
 )
@@ -53,7 +104,7 @@ setMethod("setdiff", c("GRanges", "GRanges"),
         ## If the length of a sequence is unknown (NA), then we use
         ## the max end value found on that sequence in 'x' or 'y'.
         seqlengths[is.na(seqlengths)] <-
-            maxEndPerGRangesSequence(c(x, y))[is.na(seqlengths)]
+            .maxEndPerGRangesSequence(c(x, y))[is.na(seqlengths)]
         gaps(union(gaps(x, end=seqlengths), y), end=seqlengths)
     }
 )
@@ -162,7 +213,7 @@ setMethod("pintersect", c("GRanges", "GRanges"),
         resolveStrand <- as(ansStrand == "*", "IRanges")
         if (length(resolveStrand) > 0L)
             ansStrand[as.integer(resolveStrand)] <-
-              IRanges:::extractROWS(strand(y), resolveStrand)
+              extractROWS(strand(y), resolveStrand)
         strand(x) <- ansStrand
         x
     }
@@ -246,16 +297,16 @@ setMethod("psetdiff", c("GRanges", "GRanges"),
               compatibleStrand(strand(x), strand(y))
         ## Update the ranges.
         ansRanges <- ranges(x)
-        ansRanges <- IRanges:::replaceROWS(ansRanges, ok,
-                         callGeneric(IRanges:::extractROWS(ranges(x), ok),
-                                     IRanges:::extractROWS(ranges(y), ok)))
+        ansRanges <- replaceROWS(ansRanges, ok,
+                         callGeneric(extractROWS(ranges(x), ok),
+                                     extractROWS(ranges(y), ok)))
         ranges(x) <- ansRanges
         ## Update the strand.
         ansStrand <- strand(x)
         resolveStrand <- as(ansStrand == "*", "IRanges")
         if (length(resolveStrand) > 0L)
             ansStrand[as.integer(resolveStrand)] <-
-              IRanges:::extractROWS(strand(y), resolveStrand)
+              extractROWS(strand(y), resolveStrand)
         strand(x) <- ansStrand
         x
     }
