@@ -10,26 +10,41 @@ BiocStyle::latex(use.unsrturl=FALSE)
 ### code chunk number 2: options
 ###################################################
 options(width=72)
-options("showHeadLines" = 3)
-options("showTailLines" = 3)
+options(showHeadLines=3)
+options(showTailLines=3)
+.precomputed_results_path <- "precomputed_results"
 
 
 ###################################################
-### code chunk number 3: load
+### code chunk number 3: pasillaBamSubset
 ###################################################
 library(pasillaBamSubset)
-un1 <- untreated1_chr4() ## single-end reads
+untreated1_chr4()
+untreated3_chr4()
 
 
 ###################################################
-### code chunk number 4: readGAlignments
+### code chunk number 4: pasillaBamSubset_help (eval = FALSE)
+###################################################
+## ?pasillaBamSubset
+
+
+###################################################
+### code chunk number 5: readGAlignments_1
+###################################################
+library(pasillaBamSubset)
+un1 <- untreated1_chr4()  # single-end reads
+
+
+###################################################
+### code chunk number 6: readGAlignments_2
 ###################################################
 library(GenomicAlignments)
 gal <- readGAlignments(un1)
 
 
 ###################################################
-### code chunk number 5: ScanBamParam
+### code chunk number 7: readGAlignments_3
 ###################################################
 what <- c("flag", "cigar") 
 which <- GRanges("chr4", IRanges(1, 5000)) 
@@ -40,45 +55,54 @@ neg
 
 
 ###################################################
-### code chunk number 6: readGAlignmentPairs
+### code chunk number 8: readGAlignmentPairs_1
+###################################################
+library(pasillaBamSubset)
+un3 <- untreated3_chr4()  # paired-end reads
+
+
+###################################################
+### code chunk number 9: readGAlignmentPairs_2
 ###################################################
 un3 <- untreated3_chr4()
 gapairs <- readGAlignmentPairs(un3)
 
 
 ###################################################
-### code chunk number 7: gapairs
+### code chunk number 10: readGAlignmentPairs_3
 ###################################################
 gapairs
 
 
 ###################################################
-### code chunk number 8: readGAlignmentsList
+### code chunk number 11: readGAlignmentsList_1
 ###################################################
 galist <- readGAlignmentsList(BamFile(un3, asMates=TRUE))
 
 
 ###################################################
-### code chunk number 9: galist
+### code chunk number 12: readGAlignmentsList_2
 ###################################################
 galist
 
 
 ###################################################
-### code chunk number 10: non_mates
+### code chunk number 13: readGAlignmentsList_3
 ###################################################
-non_mates <- galist[unlist(mcols(galist)$mates) == FALSE]
+non_mates <- galist[unlist(mcols(galist)$mate_status) == "unmated"]
 table(elementLengths(non_mates))
 
 
 ###################################################
-### code chunk number 11: yieldSize
+### code chunk number 14: yieldSize
 ###################################################
+library(pasillaBamSubset)
+un1 <- untreated1_chr4()
 bf <- BamFile(un1, yieldSize=100000)
 
 
 ###################################################
-### code chunk number 12: readGAlignments_by_chunk
+### code chunk number 15: readGAlignments_by_chunk
 ###################################################
 open(bf)
 cvg <- NULL
@@ -98,82 +122,184 @@ cvg
 
 
 ###################################################
-### code chunk number 13: load
+### code chunk number 16: coverage_1
 ###################################################
 library(pasillaBamSubset)
-un1 <- untreated1_chr4() ## single-end records
+un1 <- untreated1_chr4()  # single-end reads
+library(GenomicAlignments)
+reads1 <- readGAlignments(un1)
+cvg1 <- coverage(reads1)
+cvg1
 
 
 ###################################################
-### code chunk number 14: count_1
+### code chunk number 17: coverage_2
+###################################################
+cvg1$chr4
+
+
+###################################################
+### code chunk number 18: coverage_3
+###################################################
+mean(cvg1$chr4)
+max(cvg1$chr4)
+
+
+###################################################
+### code chunk number 19: peaks_1
+###################################################
+chr4_peaks <- slice(cvg1$chr4, lower=500)
+chr4_peaks
+length(chr4_peaks)  # nb of peaks
+
+
+###################################################
+### code chunk number 20: peaks_2
+###################################################
+sum(chr4_peaks)
+
+
+###################################################
+### code chunk number 21: makeTxDbFromUCSC_1 (eval = FALSE)
+###################################################
+## library(GenomicFeatures)
+## ### Internet connection required! Can take several minutes...
+## txdb <- makeTxDbFromUCSC(genome="sacCer2", tablename="ensGene")
+
+
+###################################################
+### code chunk number 22: TxDb.Hsapiens.UCSC.hg19.knownGene_1
+###################################################
+library(TxDb.Hsapiens.UCSC.hg19.knownGene)
+txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
+txdb
+
+
+###################################################
+### code chunk number 23: TxDb.Hsapiens.UCSC.hg19.knownGene_2
+###################################################
+transcripts(txdb)
+
+
+###################################################
+### code chunk number 24: makeTxDbFromBiomart_1 (eval = FALSE)
+###################################################
+## library(GenomicFeatures)
+## ### Internet connection required! Can take several minutes...
+## txdb <- makeTxDbFromBiomart(biomart="ensembl",
+##                             dataset="hsapiens_gene_ensembl")
+
+
+###################################################
+### code chunk number 25: TxDb.Athaliana.BioMart.plantsmart22_1
+###################################################
+library(TxDb.Athaliana.BioMart.plantsmart22)
+txdb <- TxDb.Athaliana.BioMart.plantsmart22
+txdb
+
+
+###################################################
+### code chunk number 26: TxDb.Athaliana.BioMart.plantsmart22_2
+###################################################
+exons(txdb)
+
+
+###################################################
+### code chunk number 27: makeTxDbFromGFF_1
+###################################################
+library(GenomicFeatures)
+gff_file <- system.file("extdata", "GFF3_files", "a.gff3",
+                        package="GenomicFeatures")
+txdb <- makeTxDbFromGFF(gff_file, format="gff3")
+txdb
+
+
+###################################################
+### code chunk number 28: makeTxDbFromGFF_2
+###################################################
+exonsBy(txdb, by="gene")
+
+
+###################################################
+### code chunk number 29: hub_1
+###################################################
+library(AnnotationHub)
+### Internet connection required!
+hub <- AnnotationHub()
+hub <- subset(hub, hub$species=='Drosophila melanogaster')
+
+
+###################################################
+### code chunk number 30: hub_2
+###################################################
+length(hub)
+head(names(hub))
+head(hub$title, n=10)
+## then look at a specific slice of the hub object.
+hub[7]
+
+
+###################################################
+### code chunk number 31: hub_3
+###################################################
+gr <- hub[[names(hub)[7]]]
+summary(gr)
+
+
+###################################################
+### code chunk number 32: hub_4
+###################################################
+metadata(gr)
+
+
+###################################################
+### code chunk number 33: hub_5
+###################################################
+txbygn <- split(gr, gr$name)
+
+
+###################################################
+### code chunk number 34: count_1
+###################################################
+library(pasillaBamSubset)
+un1 <- untreated1_chr4()  # single-end reads
+
+
+###################################################
+### code chunk number 35: count_2
 ###################################################
 library(TxDb.Dmelanogaster.UCSC.dm3.ensGene)
 exbygene <- exonsBy(TxDb.Dmelanogaster.UCSC.dm3.ensGene, "gene")
 
 
 ###################################################
-### code chunk number 15: count_2
+### code chunk number 36: count_3
 ###################################################
 library(GenomicAlignments)
 se <- summarizeOverlaps(exbygene, un1, mode="IntersectionNotEmpty")
 
 
 ###################################################
-### code chunk number 16: count_3
+### code chunk number 37: count_4
 ###################################################
 class(se)
 head(table(assays(se)$counts))
 
 
 ###################################################
-### code chunk number 17: count_4
+### code chunk number 38: count_5
 ###################################################
 identical(length(exbygene), length(assays(se)$counts))
 
 
 ###################################################
-### code chunk number 18: count_5
+### code chunk number 39: count_6
 ###################################################
-rowData(se)
+rowRanges(se)
 
 
 ###################################################
-### code chunk number 19: hub_1
-###################################################
-library(AnnotationHub)
-hub <- AnnotationHub()
-filters(hub) <- list(Species="Drosophila melanogaster")
-
-
-###################################################
-### code chunk number 20: hub_2
-###################################################
-length(hub)
-head(names(hub))
-
-
-###################################################
-### code chunk number 21: hub_3
-###################################################
-gr <- hub$goldenpath.dm3.database.ensGene_0.0.1.RData
-summary(gr)
-
-
-###################################################
-### code chunk number 22: hub_4
-###################################################
-names(metadata(gr)[[2]])
-metadata(gr)[[2]]$Tags
-
-
-###################################################
-### code chunk number 23: hub_5
-###################################################
-split(gr, gr$name)
-
-
-###################################################
-### code chunk number 24: count_table
+### code chunk number 40: count_table
 ###################################################
 library(DESeq)
 deseq <- newCountDataSet(assays(se)$counts, rownames(colData(se)))
@@ -182,70 +308,90 @@ edger <- DGEList(assays(se)$counts, group=rownames(colData(se)))
 
 
 ###################################################
-### code chunk number 25: trak_1
+### code chunk number 41: summarizeJunctions_1
+###################################################
+library(pasillaBamSubset)
+un1 <- untreated1_chr4()  # single-end reads
+library(GenomicAlignments)
+reads1 <- readGAlignments(un1)
+reads1
+
+
+###################################################
+### code chunk number 42: summarizeJunctions_2
+###################################################
+junc_summary <- summarizeJunctions(reads1)
+junc_summary
+
+
+###################################################
+### code chunk number 43: trak_1
 ###################################################
 trak2 <- "66008"
 
 
 ###################################################
-### code chunk number 26: trak_2
+### code chunk number 44: trak_2
 ###################################################
 library(TxDb.Hsapiens.UCSC.hg19.knownGene)
 txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
 
 
 ###################################################
-### code chunk number 27: trak_3
+### code chunk number 45: trak_3
 ###################################################
 library(GenomicFeatures)
-txbygene <- transcriptsBy(txdb, by="gene")[trak2]
-txbygene
+trak2_txs <- transcriptsBy(txdb, by="gene")[[trak2]]
+trak2_txs
 
 
 ###################################################
-### code chunk number 28: trak_4
+### code chunk number 46: trak_4
 ###################################################
-tx_names <- mcols(unlist(txbygene))$tx_name
-tx_names
+trak2_tx_names <- mcols(trak2_txs)$tx_name
+trak2_tx_names
 
 
 ###################################################
-### code chunk number 29: trak_5
+### code chunk number 47: trak_5
 ###################################################
-intronsbytx <- intronsByTranscript(txdb, use.names=TRUE)[tx_names]
-elementLengths(intronsbytx)
+trak2_exbytx <- exonsBy(txdb, "tx", use.names=TRUE)[trak2_tx_names]
+elementLengths(trak2_exbytx)
 
 
 ###################################################
-### code chunk number 30: trak_7
+### code chunk number 48: trak_7
 ###################################################
-exonsbytx <- exonsBy(txdb, "tx", use.names=TRUE)[tx_names]
-elementLengths(exonsbytx)
+trak2_inbytx <- intronsByTranscript(txdb, use.names=TRUE)[trak2_tx_names]
+elementLengths(trak2_inbytx)
 
 
 ###################################################
-### code chunk number 31: trak_8
+### code chunk number 49: trak_8
 ###################################################
-library(Biostrings)
 library(BSgenome.Hsapiens.UCSC.hg19)
 
 
 ###################################################
-### code chunk number 32: trak_9
+### code chunk number 50: trak_9
 ###################################################
-intron_seqs <- extractTranscriptSeqs(Hsapiens, intronsbytx)
-intron_seqs
+trak2_ex_seqs <- getSeq(Hsapiens, trak2_exbytx)
+trak2_ex_seqs
+trak2_ex_seqs[["uc002uyb.4"]]
+trak2_ex_seqs[["uc002uyc.2"]]
 
 
 ###################################################
-### code chunk number 33: trak_10
+### code chunk number 51: trak_10
 ###################################################
-exon_seqs <- extractTranscriptSeqs(Hsapiens, exonsbytx)
-exon_seqs
+trak2_in_seqs <- getSeq(Hsapiens, trak2_inbytx)
+trak2_in_seqs
+trak2_in_seqs[["uc002uyb.4"]]
+trak2_in_seqs[["uc002uyc.2"]]
 
 
 ###################################################
-### code chunk number 34: cancer_1
+### code chunk number 52: cancer_1
 ###################################################
 library(KEGG.db)
 pathways <- toTable(KEGGPATHNAME2ID)
@@ -253,7 +399,7 @@ pathways[grepl("cancer", pathways$path_name, fixed=TRUE),]
 
 
 ###################################################
-### code chunk number 35: cancer_2
+### code chunk number 53: cancer_2
 ###################################################
 library(KEGGgraph)
 dest <- tempfile()
@@ -261,7 +407,7 @@ retrieveKGML("05200", "hsa", dest, "internal")
 
 
 ###################################################
-### code chunk number 36: cancer_3
+### code chunk number 54: cancer_3
 ###################################################
 crids <- as.character(parseKGML2DataFrame(dest)[,1])
 crgenes <- unique(translateKEGGID2GeneID(crids))
@@ -269,14 +415,14 @@ head(crgenes)
 
 
 ###################################################
-### code chunk number 37: cancer_4
+### code chunk number 55: cancer_4
 ###################################################
 library(TxDb.Hsapiens.UCSC.hg19.knownGene)
 txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
 
 
 ###################################################
-### code chunk number 38: cancer_5
+### code chunk number 56: cancer_5
 ###################################################
 txbygene <- transcriptsBy(txdb, "gene")[crgenes] ## subset on colorectal genes
 map <- relist(unlist(txbygene, use.names=FALSE)$tx_id, txbygene)
@@ -284,7 +430,7 @@ map
 
 
 ###################################################
-### code chunk number 39: cancer_6
+### code chunk number 57: cancer_6
 ###################################################
 cds <- cdsBy(txdb, "tx")
 threeUTR <- threeUTRsByTranscript(txdb)
@@ -292,7 +438,7 @@ fiveUTR <- fiveUTRsByTranscript(txdb)
 
 
 ###################################################
-### code chunk number 40: cancer_7
+### code chunk number 58: cancer_7
 ###################################################
 txid <- unlist(map, use.names=FALSE)
 cds <- cds[names(cds) %in% txid]
@@ -301,7 +447,7 @@ fiveUTR <- fiveUTR[names(fiveUTR) %in% txid]
 
 
 ###################################################
-### code chunk number 41: cancer_8
+### code chunk number 59: cancer_8
 ###################################################
 length(txid) ## all possible transcripts
 length(cds)
@@ -310,20 +456,20 @@ length(fiveUTR)
 
 
 ###################################################
-### code chunk number 42: cancer_9
+### code chunk number 60: cancer_9
 ###################################################
 cds
 
 
 ###################################################
-### code chunk number 43: cancer_10
+### code chunk number 61: cancer_10
 ###################################################
 library(BSgenome.Hsapiens.UCSC.hg19)
 genome <- BSgenome.Hsapiens.UCSC.hg19
 
 
 ###################################################
-### code chunk number 44: cancer_11
+### code chunk number 62: cancer_11
 ###################################################
 threeUTR_seqs <- extractTranscriptSeqs(genome, threeUTR) 
 fiveUTR_seqs <- extractTranscriptSeqs(genome, fiveUTR) 
@@ -331,13 +477,13 @@ cds_seqs <- extractTranscriptSeqs(genome, cds)
 
 
 ###################################################
-### code chunk number 45: cancer_12
+### code chunk number 63: cancer_12
 ###################################################
 cds_seqs
 
 
 ###################################################
-### code chunk number 46: cancer_13
+### code chunk number 64: cancer_13
 ###################################################
 lst3 <- relist(threeUTR_seqs, PartitioningByWidth(sum(map %in% names(threeUTR))))
 lst5 <- relist(fiveUTR_seqs, PartitioningByWidth(sum(map %in% names(fiveUTR))))
@@ -345,14 +491,14 @@ lstc <- relist(cds_seqs, PartitioningByWidth(sum(map %in% names(cds))))
 
 
 ###################################################
-### code chunk number 47: cancer_14
+### code chunk number 65: cancer_14
 ###################################################
 length(map)
 table(elementLengths(map))
 
 
 ###################################################
-### code chunk number 48: cancer_15
+### code chunk number 66: cancer_15
 ###################################################
 table(elementLengths(lstc))
 table(elementLengths(lst3))
@@ -362,16 +508,17 @@ names(lst5)[elementLengths(lst5) == 0L] ## genes with no 5' UTR data
 
 
 ###################################################
-### code chunk number 49: cseq_1
+### code chunk number 67: cseq_1
 ###################################################
 library(Rsamtools)
 bamfile <- system.file("extdata", "ex1.bam", package="Rsamtools")
 param <- ScanBamParam(what=c("seq", "qual"))
-gal <- readGAlignmentsFromBam(bamfile, use.names=TRUE, param=param)
+library(GenomicAlignments)
+gal <- readGAlignments(bamfile, use.names=TRUE, param=param)
 
 
 ###################################################
-### code chunk number 50: cseq_2
+### code chunk number 68: cseq_2
 ###################################################
 qseq <- setNames(mcols(gal)$seq, names(gal))
 qual <- setNames(mcols(gal)$qual, names(gal))
@@ -382,7 +529,7 @@ qual_on_ref <- sequenceLayer(qual, cigar(gal),
 
 
 ###################################################
-### code chunk number 51: cseq_3
+### code chunk number 69: cseq_3
 ###################################################
 qseq_on_ref_by_chrom <- splitAsList(qseq_on_ref, seqnames(gal))
 qual_on_ref_by_chrom <- splitAsList(qual_on_ref, seqnames(gal))
@@ -390,7 +537,7 @@ pos_by_chrom <- splitAsList(start(gal), seqnames(gal))
 
 
 ###################################################
-### code chunk number 52: cseq_4
+### code chunk number 70: cseq_4
 ###################################################
 gr_by_chrom <- lapply(seqlevels(gal),
   function(seqname)
@@ -413,39 +560,39 @@ gr_by_chrom <- lapply(seqlevels(gal),
 
 
 ###################################################
-### code chunk number 53: cseq_5
+### code chunk number 71: cseq_5
 ###################################################
 gr <- do.call(c, gr_by_chrom)
 seqinfo(gr) <- seqinfo(gal)
 
 
 ###################################################
-### code chunk number 54: cseq_6
+### code chunk number 72: cseq_6
 ###################################################
 gr[1:6]
 
 
 ###################################################
-### code chunk number 55: cseq_7
+### code chunk number 73: cseq_7
 ###################################################
 qseq_on_ref
 qual_on_ref
 
 
 ###################################################
-### code chunk number 56: cseq_8
+### code chunk number 74: cseq_8
 ###################################################
 mcols(gr)$qseq_on_ref[[6]]
 
 
 ###################################################
-### code chunk number 57: cseq_9
+### code chunk number 75: cseq_9
 ###################################################
 mcols(gr)$qual_on_ref[[6]]
 
 
 ###################################################
-### code chunk number 58: cseq_10
+### code chunk number 76: cseq_10
 ###################################################
 qseq_on_ref <- mcols(gr)$qseq_on_ref
 tmp <- unlist(qseq_on_ref, use.names=FALSE)
@@ -453,20 +600,20 @@ qseq_on_ref_id <- relist(match(tmp, tmp), qseq_on_ref)
 
 
 ###################################################
-### code chunk number 59: cseq_11
+### code chunk number 77: cseq_11
 ###################################################
 qseq_on_ref_id
 
 
 ###################################################
-### code chunk number 60: cseq_12
+### code chunk number 78: cseq_12
 ###################################################
 qseq_on_ref_id2 <- endoapply(qseq_on_ref_id,
     function(ids) ids[countMatches(ids, ids) >= 0.2 * length(ids)])
 
 
 ###################################################
-### code chunk number 61: cseq_13
+### code chunk number 79: cseq_13
 ###################################################
 tmp <- unlist(qseq_on_ref_id2, use.names=FALSE)
 qseq_on_ref2 <- relist(unlist(qseq_on_ref, use.names=FALSE)[tmp],
@@ -474,7 +621,7 @@ qseq_on_ref2 <- relist(unlist(qseq_on_ref, use.names=FALSE)[tmp],
 
 
 ###################################################
-### code chunk number 62: cseq_14
+### code chunk number 80: cseq_14
 ###################################################
 split_factor <- rep.int(seqnames(gr), elementLengths(qseq_on_ref2))
 qseq_on_ref2 <- unlist(qseq_on_ref2, use.names=FALSE)
@@ -491,13 +638,13 @@ names(cm_by_chrom) <- names(qseq_pos_by_chrom)
 
 
 ###################################################
-### code chunk number 63: cseq_15
+### code chunk number 81: cseq_15
 ###################################################
 lapply(cm_by_chrom, dim)
 
 
 ###################################################
-### code chunk number 64: cseq_16
+### code chunk number 82: cseq_16
 ###################################################
 cs_by_chrom <- lapply(cm_by_chrom,
     function(cm) {
@@ -512,13 +659,13 @@ cs_by_chrom <- lapply(cm_by_chrom,
 
 
 ###################################################
-### code chunk number 65: cseq_17
+### code chunk number 83: cseq_17
 ###################################################
 cs_by_chrom
 
 
 ###################################################
-### code chunk number 66: bim_1
+### code chunk number 84: bim_1
 ###################################################
 library(BSgenome.Scerevisiae.UCSC.sacCer2)
 set.seed(22)
@@ -530,14 +677,14 @@ head(cov, 3)
 
 
 ###################################################
-### code chunk number 67: bin_2
+### code chunk number 85: bin_2
 ###################################################
 bins1 <- tileGenome(seqinfo(Scerevisiae), tilewidth=100,
                     cut.last.tile.in.chrom=TRUE)
 
 
 ###################################################
-### code chunk number 68: bin_3
+### code chunk number 86: bin_3
 ###################################################
 binnedAverage <- function(bins, numvar, mcolname)
 {
@@ -558,14 +705,14 @@ binnedAverage <- function(bins, numvar, mcolname)
 
 
 ###################################################
-### code chunk number 69: bin_4
+### code chunk number 87: bin_4
 ###################################################
 bins1 <- binnedAverage(bins1, cov, "binned_cov")
 bins1
 
 
 ###################################################
-### code chunk number 70: SessionInfo
+### code chunk number 88: SessionInfo
 ###################################################
 sessionInfo()
 
